r"""
GUI module to refine orientation and strain from Laue spots lists

Main author is J. S. Micha:   micha [at] esrf [dot] fr

version Aug 2019
from LaueTools package hosted in

https://gitlab.esrf.fr/micha/lauetools
"""

import sys
import time

import wx
import numpy as np

if wx.__version__ < "4.":
    WXPYTHON4 = False
else:
    WXPYTHON4 = True
    wx.OPEN = wx.FD_OPEN
    wx.CHANGE_DIR = wx.FD_CHANGE_DIR

    def sttip(argself, strtip):
        """alias for wxpython4
        """
        return wx.Window.SetToolTip(argself, wx.ToolTip(strtip))

    wx.Window.SetToolTipString = sttip


if sys.version_info.major == 3:
    from .. import lauecore as LT
    from .. import CrystalParameters as CP
    from .. import indexingSpotsSet as ISS
    from .. import indexingAnglesLUT as INDEX
    from .. import IOLaueTools as IOLT
    from .. import generaltools as GT
    from .. import dict_LaueTools as DictLT
    from . PlotRefineGUI import Plot_RefineFrame
    from . ResultsIndexationGUI import RecognitionResultCheckBox

else:
    import lauecore as LT
    import CrystalParameters as CP
    import indexingSpotsSet as ISS
    import indexingAnglesLUT as INDEX
    import IOLaueTools as IOLT
    import generaltools as GT
    import dict_LaueTools as DictLT
    from GUI.PlotRefineGUI import Plot_RefineFrame
    from GUI.ResultsIndexationGUI import RecognitionResultCheckBox


# --- ----------------   Classical Indexation Board
class DistanceScreeningIndexationBoard(wx.Frame):
    """
    Class of GUI for the automatic indexation board of
    a single peak list with a single material or structure
    """
    def __init__(self, parent, _id, indexation_parameters, title,
                                        StorageDict=None, DataSetObject=None):

        wx.Frame.__init__(self, parent, _id, title, size=(900, 800))

        if parent is not None:
            self.parent = parent
            self.mainframe = self.parent
        else:
            self.mainframe = self

        self.kf_direction = indexation_parameters["kf_direction"]
        self.DataPlot_filename = indexation_parameters["DataPlot_filename"]
        self.key_material = None
        self.dict_Materials = indexation_parameters["dict_Materials"]
        self.dict_Rot = indexation_parameters["dict_Rot"]
        self.list_of_cliques = indexation_parameters["Cliques"]

        self.current_exp_spot_index_list = indexation_parameters["DataToIndex"]["current_exp_spot_index_list"]
        #         print "self.current_exp_spot_index_list", self.current_exp_spot_index_list
        self.data_theta = indexation_parameters["DataToIndex"]["data_theta"]
        self.data_chi = indexation_parameters["DataToIndex"]["data_chi"]
        self.data_I = indexation_parameters["DataToIndex"]["data_I"]
        self.dataXY_exp = indexation_parameters["DataToIndex"]["dataXY"]

        self.ClassicalIndexation_Tabledist = indexation_parameters["DataToIndex"]["ClassicalIndexation_Tabledist"]

        self.defaultParam = indexation_parameters["detectorparameters"]
        self.detectordiameter = indexation_parameters["detectordiameter"]
        self.pixelsize = indexation_parameters["pixelsize"]
        self.framedim = indexation_parameters["dim"]

        self.CCDLabel = indexation_parameters["CCDLabel"]

        self.datatype = "2thetachi"

        self.CCDdetectorparameters = {}
        self.CCDdetectorparameters["CCDcalib"] = indexation_parameters["detectorparameters"]
        self.CCDdetectorparameters["framedim"] = indexation_parameters["dim"]
        self.CCDdetectorparameters["pixelsize"] = indexation_parameters["pixelsize"]
        self.CCDdetectorparameters["CCDLabel"] = indexation_parameters["CCDLabel"]
        self.CCDdetectorparameters["detectorparameters"] = indexation_parameters["detectorparameters"]
        self.CCDdetectorparameters["detectordiameter"] = indexation_parameters["detectordiameter"]

        self.IndexationParameters = indexation_parameters
        self.IndexationParameters["Filename"] = indexation_parameters["DataPlot_filename"]
        self.IndexationParameters["DataPlot_filename"] = indexation_parameters["DataPlot_filename"]
        self.IndexationParameters["current_processedgrain"] = indexation_parameters["current_processedgrain"]
        self.IndexationParameters["mainAppframe"] = indexation_parameters["mainAppframe"]
        self.IndexationParameters["indexationframe"] = self

        # print("keys of self.IndexationParameters in DistanceScreeningIndexationBoard",
        #                                     list(self.IndexationParameters.keys()))

        if self.CCDLabel in ("MARCCD165", "ROPER159", "VHR_PSI"):
            self.IndexationParameters["flipyaxis"] = True
        else:
            self.IndexationParameters["flipyaxis"] = False

        if StorageDict is None:
            self.StorageDict = {}
            self.StorageDict["mat_store_ind"] = self.mainframe.mat_store_ind
            self.StorageDict["Matrix_Store"] = self.mainframe.Matrix_Store
            self.StorageDict["dict_Rot"] = self.mainframe.dict_Rot
        else:
            self.StorageDict = StorageDict

        self.DataSet = DataSetObject

        self.list_materials = []
        self.combokeymaterial = None
        self.spotsorder = None
        self.dict_param, self.dict_param_list = None, None
        self.select_chi, self.select_theta = None, None
        self.select_dataX, self.select_dataY, self.select_I = None, None, None
        self.data = None
        self.select_dataXY = None
        self.bestmatrices = []
        self.TwicethetaChi_solution = None
        self.config_irp_filename = None

        self.initGUI()

    def initGUI(self):
        """
        GUI widgets
        """
        font3 = wx.Font(10, wx.MODERN, wx.NORMAL, wx.BOLD)

        title1 = wx.StaticText(self, -1, "Spots Selection")
        title1.SetFont(font3)
        txtcf = wx.StaticText(self, -1, "Current File:        %s   " % self.DataPlot_filename)
        nbspots_in_data = len(self.current_exp_spot_index_list)
        mssstxt = wx.StaticText(self, -1, "Spots set Size         ")
        self.nbspotmaxformatching = wx.SpinCtrl(self, -1, str(nbspots_in_data),
                                                        (60, -1), min=3, max=nbspots_in_data)

        self.setAchck = wx.CheckBox(self, -1, "")
        self.setAchck.SetValue(True)
        cstxt = wx.StaticText(self, -1, "Spots set A")
        self.spotlistA = wx.TextCtrl(self, -1, "0", size=(200, -1))
        self.sethklchck = wx.CheckBox(self, -1, "set hkl of set A spots")
        self.sethklcentral = wx.TextCtrl(self, -1, "[1,0,0]")

        self.setBchck = wx.CheckBox(self, -1, "")
        self.setBchck.SetValue(True)
        rsstxt = wx.StaticText(self, -1, "Spots Set B: ")
        self.spotlistB = wx.TextCtrl(self, -1, "to10", (200, -1))

        lutrectxt = wx.StaticText(self, -1, "Angles LUT Recognition")
        lutrectxt.SetFont(font3)

        drtatxt = wx.StaticText(self, -1, "Recognition Tol. Angle(deg)")
        self.DRTA = wx.TextCtrl(self, -1, "0.5")
        luttxt = wx.StaticText(self, -1, "LUT Nmax")
        self.nLUT = wx.SpinCtrl(self, -1, "4", (50, -1), min=3, max=7)

        elemtxt = wx.StaticText(self, -1, "Materials")
        self.SetMaterialsCombo(0)
        self.refresh = wx.Button(self, -1, "Refresh")
        self.refresh.Bind(wx.EVT_BUTTON, self.SetMaterialsCombo)
        #self.applyrulesLUT = wx.CheckBox(self, -1, "Apply Extinc. Rules")
        self.applyrulesLUT = wx.CheckBox(self, -1, "Apply Extinc. Rules")
        self.applyrulesLUT.SetValue(True)

        matchtxt = wx.StaticText(self, -1, "Matching")
        matchtxt.SetFont(font3)

        mtatxt = wx.StaticText(self, -1, "Matching Tol. Angle(deg)")
        self.MTA = wx.TextCtrl(self, -1, "0.2")
        resangtxt = wx.StaticText(self, -1, "Min. d-spacing")
        self.ResolutionAngstromctrl = wx.TextCtrl(self, -1, "False", (70, -1))

        emaxtxt = wx.StaticText(self, -1, "Energy max.: ")
        self.emax = wx.SpinCtrl(self, -1, "22", (60, -1), min=10, max=150)
        mnmstxt = wx.StaticText(self, -1, "Matching Threshold")
        self.MNMS = wx.SpinCtrl(self, -1, "15", (60, -1), min=1, max=500)

        pptxt = wx.StaticText(self, -1, "Filtering && Post Processing")
        pptxt.SetFont(font3)

        self.filterMatrix = wx.CheckBox(self, -1,
                                        "Remove equivalent Matrices (cubic symmetry)")
        self.filterMatrix.SetValue(True)
        self.verbose = wx.CheckBox(self, -1, "Print details")
        self.verbose.SetValue(False)

        self.showplotBox = wx.CheckBox(self, -1, "Plot Best result")
        self.showplotBox.SetValue(False)
        self.indexation_index = 0
        self.config_irp_filename = (self.DataPlot_filename[: -4] + "_%d.irp" % self.indexation_index)
        spcftxt= wx.StaticText(self, -1, "Saving parameters in config file")
        self.output_irp = wx.TextCtrl(self, -1, "%s" % self.config_irp_filename,
                                     size=(250, -1))

        self.StartButton = wx.Button(self, -1, "Start", size=(-1, 80))
        self.StartButton.SetFont(font3)
        quitbtn = wx.Button(self, 2, "Quit", size=(-1, 80))
        self.textprocess = wx.StaticText(self, -1, "                     ")
        self.gauge = wx.Gauge(self, -1, 1000, size=(250, 25))

        self.StartButton.Bind(wx.EVT_BUTTON, self.OnStart)
        quitbtn.Bind(wx.EVT_BUTTON, self.OnQuit)

        self.sb = self.CreateStatusBar()

        #-----  use input of cliques
        if self.list_of_cliques is not None:
            cliqueindex = 0
            self.spotlistA.SetValue(str(self.list_of_cliques[cliqueindex].tolist()))
            self.setBchck.SetValue(False)

        # layout
        h1box = wx.BoxSizer(wx.HORIZONTAL)
        h1box.Add(mssstxt,0, wx.EXPAND, 10)
        h1box.Add(self.nbspotmaxformatching,0, wx.EXPAND, 10)

        h2box = wx.BoxSizer(wx.HORIZONTAL)
        h2box.Add(self.setAchck,0, wx.EXPAND|wx.ALL, 10)
        h2box.Add(cstxt,0, wx.EXPAND|wx.ALL, 10)
        h2box.Add(self.spotlistA,0, wx.EXPAND|wx.ALL, 10)
        h2box.Add(self.sethklchck ,0, wx.EXPAND|wx.ALL, 10)
        h2box.Add(self.sethklcentral,0, wx.EXPAND|wx.ALL, 10)

        h3box = wx.BoxSizer(wx.HORIZONTAL)
        h3box.Add(self.setBchck,0, wx.EXPAND|wx.ALL, 10)
        h3box.Add(rsstxt,0, wx.EXPAND|wx.ALL, 10)
        h3box.Add(self.spotlistB,0, wx.EXPAND|wx.ALL, 10)

        h4box = wx.BoxSizer(wx.HORIZONTAL)
        h4box.Add(drtatxt,0, wx.EXPAND|wx.ALL, 10)
        h4box.Add(self.DRTA,0, wx.EXPAND|wx.ALL, 10)
        h4box.Add(luttxt,0, wx.EXPAND|wx.ALL, 10)
        h4box.Add(self.nLUT,0, wx.EXPAND|wx.ALL, 10)

        h5box = wx.BoxSizer(wx.HORIZONTAL)
        h5box.Add(elemtxt,0, wx.EXPAND|wx.ALL, 10)
        h5box.Add(self.combokeymaterial,0, wx.EXPAND|wx.ALL, 10)
        h5box.Add(self.refresh,0, wx.EXPAND|wx.ALL, 10)
        h5box.Add(self.applyrulesLUT,0, wx.EXPAND|wx.ALL, 10)

        h6box = wx.BoxSizer(wx.HORIZONTAL)
        h6box.Add(mtatxt,0, wx.EXPAND|wx.ALL, 10)
        h6box.Add(self.MTA,0, wx.EXPAND|wx.ALL, 10)
        h6box.Add(emaxtxt,0, wx.EXPAND|wx.ALL, 10)
        h6box.Add(self.emax,0, wx.EXPAND|wx.ALL, 10)

        h7box = wx.BoxSizer(wx.HORIZONTAL)
        h7box.Add(resangtxt,0, wx.EXPAND|wx.ALL, 10)
        h7box.Add(self.ResolutionAngstromctrl,0, wx.EXPAND|wx.ALL, 10)
        h7box.Add(mnmstxt,0, wx.EXPAND|wx.ALL, 10)
        h7box.Add(self.MNMS,0, wx.EXPAND|wx.ALL, 10)

        h8box = wx.BoxSizer(wx.HORIZONTAL)
        h8box.Add(self.filterMatrix,0, wx.EXPAND, 10)
        h8box.Add(self.verbose,0, wx.EXPAND, 10)

        h9box = wx.BoxSizer(wx.HORIZONTAL)
        h9box.Add(spcftxt,0, wx.EXPAND, 10)
        h9box.Add(self.output_irp,0, wx.EXPAND, 10)

        h10box = wx.BoxSizer(wx.HORIZONTAL)
        h10box.Add(self.StartButton,1, wx.EXPAND, 10)
        h10box.Add(quitbtn,0, wx.EXPAND, 10)
        h10box.Add(self.textprocess,0, wx.EXPAND, 10)
        h10box.Add(self.gauge,0, wx.EXPAND, 10)

        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(txtcf,0, wx.EXPAND, 10)
        vbox.Add(h1box,0, wx.EXPAND, 10)
        vbox.AddSpacer(10)
        vbox.Add(title1,0, wx.EXPAND, 10)
        vbox.Add(h2box,0, wx.EXPAND, 10)
        vbox.Add(h3box,0, wx.EXPAND, 10)
        vbox.AddSpacer(10)
        vbox.Add(lutrectxt,0, wx.EXPAND, 10)
        vbox.Add(h4box,0, wx.EXPAND, 10)
        vbox.Add(h5box,0, wx.EXPAND, 10)
        vbox.AddSpacer(10)
        vbox.Add(matchtxt,0, wx.EXPAND, 10)
        vbox.Add(h6box,0, wx.EXPAND, 10)
        vbox.Add(h7box,0, wx.EXPAND, 10)
        vbox.AddSpacer(10)
        vbox.Add(pptxt,0, wx.EXPAND, 10)
        vbox.Add(h8box,0, wx.EXPAND, 10)
        vbox.Add(self.showplotBox,0, wx.EXPAND, 10)
        vbox.Add(h9box,0, wx.EXPAND, 10)
        vbox.Add(h10box,0, wx.EXPAND, 10)

        self.SetSizer(vbox)

        self.Show(True)
        self.Centre()

        # tooltips
        emaxtp = "Maximum energy of the white beam bandpass in keV. The higher this number the larger the number of simulated spots"
        emaxtxt.SetToolTipString(emaxtp)
        self.emax.SetToolTipString(emaxtp)

        resangtp = "Raw crystal lattice spacing resolution in Angstrom. HKL spots with lattice spacing smaller than this value are not simulated.\n"
        resangtp += "When crystal disorder occurs spots with low lattice spacings (or large hkl's) are weaks"
        resangtxt.SetToolTipString(resangtp)
        self.ResolutionAngstromctrl.SetToolTipString(resangtp)

        elemtip = "Select reference structure from the Material or Element list."
        self.combokeymaterial.SetToolTipString(elemtip)
        elemtxt.SetToolTipString(elemtip)

        self.refresh.SetToolTipString(
            "Refresh the Material list to admit new materials built in other LaueToolsGUI menus.")

        luttip = "Choose the largest hkl index to build the reference angular distance Looking up Table (LUT). Given n, the LUT contains all mutual angles between normals of lattice planes from (0,0,1) to (n,n,n-1) types"
        luttxt.SetToolTipString(luttip)
        self.nLUT.SetToolTipString(luttip)

        tsstip = "If checked, set of spot index to compute all mutual angles between spots of setA and setB."
        rsstxt.SetToolTipString(tsstip)
        self.spotlistB.SetToolTipString(tsstip)

        mssstip = "Number of experimental spots used find the best matching with the Laue Pattern simulated according to a recognised angle in LUT."
        mssstxt.SetToolTipString(mssstip)
        self.nbspotmaxformatching.SetToolTipString(mssstip)

        cstip = 'Experimental spot index (integer), list of spot indices to be considered as central spots, OR for example "to12" meaning spot indices ranging from 0 to 12 (included). All mutual angles between spots of setA will be considered for recognition. If setB is checked, all mutual angles between spots of setA and spots of setB will be calculated and compared to angles in reference LUT for recognition.\n'
        # cstip += 'List of spots must written in with bracket (e.g. [0,1,2,5,8]). Central spots index must be strictly lower to nb of spots of the "recognition set".'
        cstxt.SetToolTipString(cstip)
        self.spotlistA.SetToolTipString(cstip)

        drtatip = "Tolerance angle (in degree) within which an experimental angle (between a central and a recognition set spot) must be close to a reference angle in LUT to be used for simulation the Laue pattern (to be matched to the experimenal one)."
        drtatxt.SetToolTipString(drtatip)
        self.DRTA.SetToolTipString(drtatip)

        mtatip = "Tolerance angle (in degree) within which an experimental and simulated spots are close enough to be considered as matched."
        mtatxt.SetToolTipString(mtatip)
        self.MTA.SetToolTipString(mtatip)

        mnmstip = "Minimum number of matched spots (experimental with simulated one) to display the matching results of a possible indexation solution."
        mnmstxt.SetToolTipString(mnmstip)
        self.MNMS.SetToolTipString(mnmstip)

        self.showplotBox.SetToolTipString('Plot all exp. and theo. Laue Patterns for which the '
                        'number of matched spots is larger than "Minimum Number Matched spots".')

        self.filterMatrix.SetToolTipString("Keep only one orientation matrix for matrices which "
                                "are equivalent (cubic symmetry unit cell vectors permutations).")

        sethkltip = "Set the [h,k,l] Miller indices of central spot. This will reduce the running time of recognition."
        self.sethklchck.SetToolTipString(sethkltip)
        self.sethklcentral.SetToolTipString(sethkltip)

        self.verbose.SetToolTipString("Display details for long indexation procedure")

        self.applyrulesLUT.SetToolTipString("Apply systematic lattice extinction Rules "
            "when calculating angles LUT from reciprocal directions. "
            "To index single grain high hkl spots, better uncheck this (e.g. back reflection or "
            "transmission geometry.")

    def SetMaterialsCombo(self, _):
        """ set material combo  from   self.dict_Materials
        .. todo:: better to use gridsizer and refresh/update of combo
        """
        self.list_materials = sorted(self.dict_Materials.keys())

        self.combokeymaterial = wx.ComboBox(self, -1, "Ge", (140, 170), size=(150, -1),
                                        choices=self.list_materials, style=wx.CB_READONLY)

        self.combokeymaterial.Bind(wx.EVT_COMBOBOX, self.EnterCombokeymaterial)

    def EnterCombokeymaterial(self, event):
        """
        in classicalindexation
        """
        item = event.GetSelection()
        self.key_material = self.list_materials[item]
        self.filterMatrix.SetValue(CP.hasCubicSymmetry(self.key_material, dictmaterials=self.dict_Materials))

        self.sb.SetStatusText("Selected material: %s" % str(self.dict_Materials[self.key_material]))
        event.Skip()

    def getparams_for_irpfile(self):
        """get indexation and refine parameters to be written in a .irp file

        :return: boolean for success
        """

        # first element is omitted
        List_options = ISS.LIST_OPTIONS_INDEXREFINE[1:]
        # save config file .irp
        #         LIST_OPTIONS_INDEXREFINE =['nb materials',
        #                             'key material',
        #                              'nbGrainstoFind',
        #                              'emin',
        #                             'emax',
        #                             'MATCHINGRATE THRESHOLD IAL',
        #                             'AngleTolLUT',
        #                             'MATCHINGRATE ANGLE TOL',
        #                             'NBMAXPROBED',
        #                             'MinimumNumberMatches',
        #                             'central spots indices',
        #                             'ResolutionAngstrom',
        #                             'nLUTmax',
        #                             'setCentralSpotsHKL',
        #                             'UseIntensityWeights',
        #                             'nbSpotsToIndex',
        #                             'List Matching Tol Angles',
        #                             'Spots Order Reference File']

        sethklcentral = "None"
        self.spotsorder = "None"
        if self.sethklchck.GetValue():
            sethklcentral = str(self.sethklcentral.GetValue())

        MatchingAngleTol = float(self.MTA.GetValue())

        nbSpotsToIndex = 1000

        List_Ctrls = [self.combokeymaterial, 1, 5.0, self.emax, 100.0,
                        self.DRTA, MatchingAngleTol, self.spotlistB, 6,
                        self.spotlistA, self.ResolutionAngstromctrl, self.nLUT,
                        sethklcentral, True, nbSpotsToIndex,
                        [MatchingAngleTol, MatchingAngleTol / 2.0],
                        self.spotsorder]

        self.dict_param = {}
        flag = True
        # print("len(List_options)", len(List_options))
        # print("len(List_Ctrls)", len(List_Ctrls))

        for kk, option_key in enumerate(List_options):
            if not isinstance(List_Ctrls[kk], (int, str, list, float, bool)):
                val = str(List_Ctrls[kk].GetValue())
            else:
                val = List_Ctrls[kk]

            self.dict_param[option_key] = val

        self.dict_param_list = [self.dict_param]

        # print("self.dict_param_list", self.dict_param_list)

        return flag

    def Save_irp_configfile(self, outputfile="mytest_irp.irp"):
        """
        save indexation parameters in .irp file
        """
        ISS.saveIndexRefineConfigFile(self.dict_param_list, outputfilename=outputfile)

    def readspotssetctrl(self, txtctrl):
        """read, parse a spotset txtctrl
        """
        spot_list = txtctrl.GetValue()
        israngefromzero = False
        if spot_list[0] != "-":
            # print "coucou"
            # this a list of spots
            if spot_list.startswith("["):
                # print "coucou2"
                spot_index_central = str(spot_list)[1:-1].split(",")
                # print spot_index_central
                arr_index = np.array(spot_index_central)

                # print np.array(arr_index, dtype = int)
                spot_index_central = list(np.array(arr_index, dtype=int))
                nb_central_spots = len(spot_index_central)

            # this is range from 0 to a spot index
            elif spot_list.startswith("to"):
                spot_index_central = list(range(int(spot_list[2:]) + 1))
                nb_central_spots = len(spot_index_central)
                israngefromzero = True

            #this is a single spot index (integer)
            else:
                spot_index_central = int(spot_list)
                nb_central_spots = 1

        else:  # minus in front of integer
            spot_index_central = 0
            nb_central_spots = 1

        return spot_index_central, nb_central_spots, israngefromzero

    def parse_spotssetctrls(self):
        """
        parse txtctrls of spotsset A and B
        """
        #----------   Spots set Selection for mutual angle computation
        spotsB = None

        spotsA, nbA, _ = self.readspotssetctrl(self.spotlistA)

        if nbA == 1:
            maxindA = spotsA
        else:
            maxindA = max(spotsA)

        #spotB is not checked
        if not self.setBchck.GetValue():
            if nbA == 1:
                wx.MessageBox("if only spots set A is checked, you must provide a set of spots "
                "by filling 'to5' or '[5,1,4,3]'", "Error")

            nbmax_probed = maxindA+1
            spot_index_central = spotsA
            nb_central_spots = nbA
            # this is a range set
            if (nbA-1) == maxindA:
                spotssettype = 'rangeset'
            # this is a list of spots
            else:
                spotssettype = 'listsetA'

            spotsB = np.arange(0, nbmax_probed)

        #spotB is checked
        else:
            spotsB, nbB, israngeB = self.readspotssetctrl(self.spotlistB)
            nbmax_probed = nbB
            spot_index_central = spotsA
            nb_central_spots = nbA
            # case of [5,3,6,17] with B = to18
            if maxindA < nbmax_probed and israngeB:
                spotssettype = 'rangeset'
            # case of [5,3,6,17] with B = to5
            else:
                spotssettype = 'listsetAB'

        return spotssettype, spot_index_central, nb_central_spots, nbmax_probed, spotsB


    def OnStart(self, _):
        """
        starts automatic (classical) indexation:

        Recognition is based on the angular distance between two spots from a set of distances
        """
        t0 = time.time()

        energy_max = int(self.emax.GetValue())

        ResolutionAngstrom = self.ResolutionAngstromctrl.GetValue()
        if ResolutionAngstrom == "False":
            ResolutionAngstrom = False
        else:
            ResolutionAngstrom = float(ResolutionAngstrom)
        print("ResolutionAngstrom in OnStart Classical indexation", ResolutionAngstrom)

        self.key_material = str(self.combokeymaterial.GetValue())
        latticeparams = self.dict_Materials[self.key_material][1]
        B = CP.calc_B_RR(latticeparams)

        # read maximum index of hkl for building angles Look Up Table(LUT)
        nLUT = self.nLUT.GetValue()
        try:
            n = int(nLUT)
            if n > 7:
                wx.MessageBox("! LUT Nmax is too high!\n This value is set to 7 ", "INFO")
            elif n < 1:
                wx.MessageBox("! LUT Nmax is not positive!\n This value is set to 1 ", "INFO")
            n = min(7, n)
            n = max(1, n)
        except ValueError:
            print("!!  maximum index for building LUT is not an integer   !!!")
            wx.MessageBox("! LUT Nmax is not an integer!\n This value is set to 3 ", "INFO")
            n = 3

        rough_tolangle = float(self.DRTA.GetValue())
        fine_tolangle = float(self.MTA.GetValue())
        Minimum_MatchesNb = int(self.MNMS.GetValue())
        #         print "Recognition tolerance angle ", rough_tolangle
        #         print "Matching tolerance angle ", fine_tolangle

        #----------   Spots set Selection for mutual angle computation
        (spotssettype, spot_index_central, nb_central_spots,
                                nbmax_probed, spotsB) = self.parse_spotssetctrls()
        print("--spotssettype --#\n\n    ", self.parse_spotssetctrls(), "      \n\n*")

        # TODO spot_index_central and spotsB to be combined to find UBS
        #------------------------------------------------

        # whole exp.data spots dict
        #         self.IndexationParameters['AllDataToIndex']
        self.data_theta = self.IndexationParameters["AllDataToIndex"]["data_theta"]
        self.data_chi = self.IndexationParameters["AllDataToIndex"]["data_chi"]
        self.data_I = self.IndexationParameters["AllDataToIndex"]["data_I"]
        self.dataXY_exp = (self.IndexationParameters["AllDataToIndex"]["data_pixX"],
                            self.IndexationParameters["AllDataToIndex"]["data_pixY"])

        # there is no precomputed angular distances between spots
        if not self.ClassicalIndexation_Tabledist:
            # Selection of spots among the whole data
            # MSSS number
            MatchingSpotSetSize = int(self.nbspotmaxformatching.GetValue())

            # select 1rstly spots that have not been indexed and 2ndly reduced list by user
            index_to_select = np.take(self.current_exp_spot_index_list,
                                        np.arange(MatchingSpotSetSize))

            self.select_theta = self.data_theta[index_to_select]
            self.select_chi = self.data_chi[index_to_select]
            self.select_I = self.data_I[index_to_select]

            # print("index_to_select", index_to_select)
            # print("len self.dataXY_exp[0]", len(self.dataXY_exp[0]))

            self.select_dataX = self.dataXY_exp[0][index_to_select]
            self.select_dataY = self.dataXY_exp[1][index_to_select]
            # print select_theta
            # print select_chi
            if spotssettype in ("rangeset", ):
                listcouple = np.array([self.select_theta, self.select_chi]).T
                # compute angles between spots
                Tabledistance = GT.calculdist_from_thetachi(listcouple, listcouple)

            #             # with CYTHON
            #             import angulardist
            #             listcouple = np.array([2.*self.select_theta, self.select_chi]).T
            #             lc = listcouple.copy(order='c')
            #             Tabledistance = angulardist.calculdist_from_2thetachi(lc, lc)
        else:
            print("Preset Tabledistance is Not implemented !")
            return

        self.data = (2 * self.select_theta, self.select_chi, self.select_I, self.DataPlot_filename)

        self.select_dataXY = (self.select_dataX, self.select_dataY)

        # detector geometry
        detectorparameters = {}
        detectorparameters["kf_direction"] = self.kf_direction
        detectorparameters["detectorparameters"] = self.defaultParam
        detectorparameters["detectordiameter"] = self.detectordiameter
        detectorparameters["pixelsize"] = self.pixelsize
        detectorparameters["dim"] = self.framedim

        restrictLUT_cubicSymmetry = True
        set_central_spots_hkl = None

        if self.sethklchck.GetValue():
            strhkl = str(self.sethklcentral.GetValue())[1:-1].split(",")

            if not self.spotlistB.GetValue():
                wx.MessageBox('Please check Spots Set B', 'INFO')

            H, K, L = strhkl
            H, K, L = int(H), int(K), int(L)
            # LUT with cubic symmetry does not have negative L
            if L < 0:
                restrictLUT_cubicSymmetry = False

            set_central_spots_hkl = [[int(H), int(K), int(L)]]

        # restrict LUT if allowed and if crystal is cubic
        restrictLUT_cubicSymmetry = restrictLUT_cubicSymmetry and CP.hasCubicSymmetry(
            self.key_material, dictmaterials=self.dict_Materials)

        print("set_central_spots_hkl", set_central_spots_hkl)
        print("restrictLUT_cubicSymmetry", restrictLUT_cubicSymmetry)

        LUT_with_rules = self.applyrulesLUT.GetValue()

        self.getparams_for_irpfile()
        self.Save_irp_configfile(outputfile=self.output_irp.GetValue())

        self.textprocess.SetLabel("Processing Indexation")
        self.gauge.SetRange(nbmax_probed * nb_central_spots)

        #         import longtask as LTask
        #
        #         taskboard = LTask.MainFrame(None, -1, INDEX.getOrientMatrices,
        #                                        [spot_index_central,
        #                                     energy_max,
        #                                     Tabledistance[:nbmax_probed, :nbmax_probed],
        #                                     self.select_theta, self.select_chi],
        #                                     (n,
        #                                     ResolutionAngstrom,
        #                                     B,
        #                                     restrictLUT_cubicSymmetry,
        #                                     None,
        #                                     rough_tolangle,
        #                                     fine_tolangle,
        #                                     Minimum_MatchesNb,
        #                                     self.key_material,
        #                                     0,
        #                                     3,
        #                                     0,
        #                                     None,  # addmatrix
        #                                     0,  # verbose
        #                                     detectorparameters,
        #                                     set_central_spots_hkl,
        #                                     verbosedetails,
        #                                     None))  # actually will be overwritten by self of MainFrame
        #
        #         taskboard.Show(True)
        #
        #         return

        # autoindexation core procedure
        # print("self.IndexationParameters['dict_Materials']",self.IndexationParameters['dict_Materials']
        excludespotspairs = [[0, 0]]
        print('spotssettype', spotssettype)
        if spotssettype in ("rangeset", ):
            res = INDEX.getOrientMatrices(spot_index_central,
                                    energy_max,
                                    Tabledistance[:nbmax_probed, :nbmax_probed],
                                    self.select_theta,
                                    self.select_chi,
                                    n=n,
                                    ResolutionAngstrom=ResolutionAngstrom,
                                    B=B,
                                    cubicSymmetry=restrictLUT_cubicSymmetry,
                                    LUT=None,
                                    LUT_tol_angle=rough_tolangle,
                                    MR_tol_angle=fine_tolangle,
                                    Minimum_Nb_Matches=Minimum_MatchesNb,
                                    key_material=self.key_material,
                                    plot=0,
                                    verbose=0,
                                    detectorparameters=detectorparameters,
                                    addMatrix=None,  # To add a priori good candidates...
                                    set_central_spots_hkl=set_central_spots_hkl,
                                    verbosedetails=1,  # verbosedetails,
                                    gauge=self.gauge,
                                    dictmaterials=self.IndexationParameters['dict_Materials'],
                                    MaxRadiusHKL=False,#True could be OK for this workflow
                                    LUT_with_rules=LUT_with_rules,
                                    excludespotspairs=excludespotspairs)

        elif spotssettype in ('listsetA', 'listsetAB', ):
            # and spotsB is checked
            if spotssettype in ('listsetA', ):
                spotsB = spot_index_central
            print('arguments of  getOrientMatrices_fromTwoSets()')
            print("--->",spot_index_central, spotsB,
                                                energy_max, self.select_theta, self.select_chi,
                                                n, self.key_material, rough_tolangle,
                                                detectorparameters,
                                                set_central_spots_hkl,
                                                Minimum_MatchesNb,
                                                LUT_with_rules,
                                                excludespotspairs)
            res = INDEX.getOrientMatrices_fromTwoSets(spot_index_central, spotsB,
                                                energy_max, self.select_theta, self.select_chi,
                                                n, self.key_material, rough_tolangle,
                                                detectorparameters,
                                                set_hkl_1=set_central_spots_hkl,
                                                minimumNbMatches=Minimum_MatchesNb,
                                                LUT_with_rules=LUT_with_rules,
                                                excludespotspairs=excludespotspairs)

        if len(res[0]) > 0:
            self.bestmatrices, stats_res = res
            print('getOrientMatrices_SubSpotsSets found %d solutions', len(res[0]))
        else:
            wx.MessageBox('Sorry! Nothing found !!\nTry to increase nLUT or the nb of spots '
                                                            'probed in spots sets A and B')
            return

        # Update DataSet Object
        if self.DataSet is None:
            print("self.bestmatrices, stats_res")
            print(self.bestmatrices, stats_res)
            return

        self.DataSet.key_material = self.key_material
        self.DataSet.emin = 5
        self.DataSet.emax = energy_max

        self.textprocess.SetLabel("Indexation Completed")

        print("General stats_res before filtering and removing duplicates", stats_res)
        nb_solutions = len(self.bestmatrices)

        keep_only_equivalent = CP.isCubic(DictLT.dict_Materials[self.key_material][1])

        if set_central_spots_hkl not in (None, [None]):
            keep_only_equivalent = False

        # print("self.bestmatrices before")
        for ra, ub in enumerate(self.bestmatrices):
            print("\nrank : %d" % ra)
            print(ub)
        if nb_solutions > 1:
            print("Merging matrices")
            # print("keep_only_equivalent = %s" % keep_only_equivalent)
            self.bestmatrices, stats_res = ISS.MergeSortand_RemoveDuplicates(
                                                        self.bestmatrices,
                                                        stats_res,
                                                        Minimum_MatchesNb,
                                                        tol=0.005,
                                                        keep_only_equivalent=keep_only_equivalent,
                                                    )

        print("stats_res", stats_res)
        nb_solutions = len(self.bestmatrices)
        # print("self.bestmatrices after")
        # for ra, ub in enumerate(self.bestmatrices):
        #     print("\nrank : %d" % ra)
        #     print(ub)
        computingtime = time.time()-t0
        print('Computing time ===> %.2f' % computingtime)

        print("spot_index_central", spot_index_central)

        if nb_solutions:
            print("%d matrice(s) found" % nb_solutions)
            print("self.bestmatrices")
            print(self.bestmatrices)
            print("\nEach Matrix is stored in 'MatIndex_#' for further simulation")
            for k in range(nb_solutions):
                self.dict_Rot["MatIndex_%d" % (k + 1)] = self.bestmatrices[k]

            stats_properformat = []
            for elem in stats_res:
                elem[0] = int(elem[0])
                elem[1] = int(elem[1])
                stats_properformat.append(tuple(elem))

            simulparameters = {}
            simulparameters["detectordiameter"] = self.detectordiameter
            simulparameters["kf_direction"] = self.kf_direction
            simulparameters["detectordistance"] = self.defaultParam[0]
            simulparameters["pixelsize"] = self.pixelsize

            # a single central point were used for distance recognition
            if nb_central_spots == 1:

                self.TwicethetaChi_solution = [0 for k_solution in range(nb_solutions)]
                paramsimul = []

                emax = int(self.emax.GetValue())
                emin = 5

                for k_solution in range(nb_solutions):

                    orientmatrix = self.bestmatrices[k_solution]

                    # only orientmatrix, self.key_material are used ----------------------
                    vecteurref = np.eye(3)  # means: a* // X, b* // Y, c* //Z
                    # old definition of grain
                    grain = [vecteurref, [1, 1, 1], orientmatrix, self.key_material]
                    # ------------------------------------------------------------------

                    # normally in this method fastcompute = 1, gives 2theta, chi
                    TwicethetaChi = LT.SimulateResult(grain, emin, emax,
                                                        simulparameters,
                                                        ResolutionAngstrom=ResolutionAngstrom,
                                                        fastcompute=1,
                                                        dictmaterials=self.IndexationParameters['dict_Materials'])
                    self.TwicethetaChi_solution[k_solution] = TwicethetaChi

                    paramsimul.append((grain, emin, emax))

                    # to plot best results
                    if self.showplotBox.GetValue():

                        print("Plotting result for emin, emax = %.2f,%.2f"
                            % (emin, int(self.emax.GetValue())))
                        print("#central spot: %d" % spot_index_central)

                        title = ("Classical Indexation Result Plot :  #central spot: %d  solution # %d"
                            % (spot_index_central, k_solution))

                        plotresult = Plot_RefineFrame(self,
                                                        -1,
                                                        title,
                                                        datatype="2thetachi",
                                                        key_material=self.key_material,
                                                        kf_direction=self.kf_direction,
                                                        Params_to_simulPattern=(grain, emin, energy_max),
                                                        ResolutionAngstrom=ResolutionAngstrom,
                                                        MATR=fine_tolangle,
                                                        CCDdetectorparameters=self.CCDdetectorparameters,
                                                        IndexationParameters=self.IndexationParameters,
                                                        StorageDict=self.StorageDict,
                                                        DataSetObject=self.DataSet)

                        plotresult.Show(True)

            # many central points were used for distance recognition
            elif nb_central_spots > 1:

                nb_to_plot = nb_solutions

                emin = 5

                print("Plotting result for emin, emax = %.2f,%.2f"
                    % (emin, int(self.emax.GetValue())))
                self.TwicethetaChi_solution = [0 for m in range(nb_to_plot)]
                paramsimul = []

                emax = float(self.emax.GetValue())

                for m in range(nb_to_plot):

                    orientmatrix = self.bestmatrices[m]

                    # only orientmatrix, self.key_material are used ----------------------
                    vecteurref = np.eye(3)
                    # old definition of grain
                    grain = [vecteurref, [1, 1, 1], orientmatrix, self.key_material]

                    # update of grain definition is done in SimulateResult()
                    # ------------------------------------------------------------------

                    TwicethetaChi = LT.SimulateResult(grain,
                                                        5,
                                                        emax,
                                                        simulparameters,
                                                        ResolutionAngstrom=ResolutionAngstrom,
                                                        fastcompute=1)

                    self.TwicethetaChi_solution[m] = TwicethetaChi
                    emax = int(self.emax.GetValue())
                    paramsimul.append((grain, emin, emax))

                    if self.showplotBox.GetValue():  # to plot best results selected a priori by user
                        title = "Classical Indexation Result Plot"

                        plotresult = Plot_RefineFrame(self,
                                                -1,
                                                title,
                                                data_added=[TwicethetaChi[0], -TwicethetaChi[1]],
                                                kf_direction=self.kf_direction,
                                                datatype="2thetachi",
                                                key_material=self.key_material,
                                                Params_to_simulPattern=(grain, emin, self.emax.GetValue()),
                                                ResolutionAngstrom=ResolutionAngstrom,
                                                CCDdetectorparameters=self.CCDdetectorparameters,
                                                IndexationParameters=self.IndexationParameters,
                                                StorageDict=self.StorageDict,
                                                DataSetObject=self.DataSet)

                        plotresult.Show(True)

            self.IndexationParameters["paramsimul"] = paramsimul
            self.IndexationParameters["bestmatrices"] = self.bestmatrices
            self.IndexationParameters["TwicethetaChi_solutions"] = self.TwicethetaChi_solution
            # display "statistical" results
            RRCBClassical = RecognitionResultCheckBox(self, -1,
                                                    "Screening Distances Indexation Solutions",
                                                    stats_properformat,
                                                    self.data,
                                                    rough_tolangle,
                                                    fine_tolangle,
                                                    key_material=self.key_material,
                                                    emax=emax,
                                                    ResolutionAngstrom=ResolutionAngstrom,
                                                    kf_direction=self.kf_direction,
                                                    datatype="2thetachi",
                                                    data_2thetachi=(2.0 * self.select_theta, self.select_chi),
                                                    data_XY=self.select_dataXY,
                                                    CCDdetectorparameters=self.CCDdetectorparameters,
                                                    IndexationParameters=self.IndexationParameters,
                                                    StorageDict=self.StorageDict,
                                                    mainframe="billframerc",  # self.mainframe
                                                    DataSetObject=self.DataSet)

            RRCBClassical.Show(True)

            self.indexation_index += 1
            pos_ = self.config_irp_filename[::-1].find("_")
            pos_ = len(self.config_irp_filename) - (pos_ + 1)
            self.config_irp_filename = (
                self.config_irp_filename[:pos_] + "_%d.irp" % self.indexation_index)
            self.output_irp.SetValue(self.config_irp_filename)

        else:  # any matrix was found
            print("!!  Nothing found   !!!")
            wx.MessageBox(
                "! NOTHING FOUND !\nTry to reduce the Minimum Number Matched Spots to catch something!",
                "INFO")

    def OnQuit(self, _):
        """ quit
        """
        self.Close()

if __name__ == "__main__":
    filename = "/home/micha/LaueTools/Examples/Ge/dat_Ge0001.cor"

    (Current_peak_data,
        data_theta,
        data_chi,
        data_pixX,
        data_pixY,
        data_I,
        calib,
        CCDCalibDict,
    ) = IOLT.readfile_cor(filename, output_CCDparamsdict=True)

    indexation_parameters = {}
    indexation_parameters["kf_direction"] = "Z>0"
    indexation_parameters["DataPlot_filename"] = filename
    indexation_parameters["dict_Materials"] = DictLT.dict_Materials
    indexation_parameters["DataToIndex"] = {}
    indexation_parameters["DataToIndex"]["data_theta"] = data_theta
    indexation_parameters["DataToIndex"]["data_chi"] = data_chi
    indexation_parameters["DataToIndex"]["dataXY"] = data_pixX, data_pixY
    indexation_parameters["DataToIndex"]["data_I"] = data_I
    indexation_parameters["DataToIndex"]["current_exp_spot_index_list"] = np.arange(len(data_theta))
    indexation_parameters["DataToIndex"]["ClassicalIndexation_Tabledist"] = None
    indexation_parameters["current_processedgrain"] = 0
    indexation_parameters["dict_Rot"] = DictLT.dict_Rot
    indexation_parameters["index_foundgrain"] = 0
    indexation_parameters["detectordiameter"] = 165.0
    indexation_parameters["pixelsize"] = 165.0 / 2048
    indexation_parameters["dim"] = (2048, 2048)
    indexation_parameters["detectorparameters"] = calib
    indexation_parameters["CCDLabel"] = "MARCCD165"

    indexation_parameters["mainAppframe"] = None

    StorageDict = {}
    StorageDict["mat_store_ind"] = 0
    StorageDict["Matrix_Store"] = []
    StorageDict["dict_Rot"] = DictLT.dict_Rot
    StorageDict["dict_Materials"] = DictLT.dict_Materials

    AIGUIApp = wx.App()
    AIGUIframe = DistanceScreeningIndexationBoard(None,
                                                    -1,
                                                    indexation_parameters,
                                                    "test automatic indexation",
                                                    StorageDict=StorageDict,
                                                    DataSetObject=None)
    AIGUIframe.Show()
    AIGUIApp.MainLoop()



from __future__ import print_function
"""
module of lauetools project

JS Micha May 2019
"""
import sys
#import os

# sys.path.insert(0, os.path.abspath('../..'))
# print('sys.path in CrystalParameters', sys.path)

import numpy as np

if sys.version_info.major == 3:
    from . import dict_LaueTools as DictLT
    from . import generaltools as GT
    from . import LaueGeometry as LaueGeom
    from . CrystalParameters import matstarlab_to_matstarlabOND, norme
    from . import findorient as FO

else:
    import generaltools as GT
    import dict_LaueTools as DictLT
    import LaueGeometry as LaueGeom
    import findorient as FO

DEG = np.pi / 180.0

NORMAL_TO_SAMPLE_AXIS = np.array([-np.sin(40.0 * DEG), 0, np.cos(40.0 * DEG)])


# --- -------------------  Mapping visualization
def calc_Euler_angles(mat3x3):
    r"""
    Calculates unique 3 euler angles representation of mat3x3

    .. note::
        from O Robach

    .. todo:: to be placed in generaltools
    """
    # mat3x3 = matrix "minimized" in LT lab frame
    # see F2TC.find_lowest_Euler_Angles_matrix
    # phi 0, theta 1, psi 2

    mat = GT.matline_to_mat3x3(
        matstarlab_to_matstarlabOND(GT.mat3x3_to_matline(mat3x3)))

    RAD = 180.0 / np.pi

    euler = np.zeros(3, float)
    euler[1] = RAD * np.arccos(mat[2, 2])

    if np.abs(np.abs(mat[2, 2]) - 1.0) < 1e-5:
        # if theta is zero, phi+psi is defined, if theta is pi, phi-psi is defined */
        # put psi = 0 and calculate phi */
        # psi */
        euler[2] = 0.0
        # phi */
        euler[0] = RAD * np.arccos(mat[0, 0])
        if mat[0, 1] < 0.0:
            euler[0] = -euler[0]
    else:
        # psi */
        toto = np.sqrt(1 - mat[2, 2] * mat[2, 2])  # sin theta - >0 */
        euler[2] = RAD * np.arccos(mat[1, 2] / toto)
        # phi */
        euler[0] = RAD * np.arccos(-mat[2, 1] / toto)
        if mat[2, 0] < 0.0:
            euler[0] = 360.0 - euler[0]

        # print "Euler angles phi theta psi (deg)"
        # print euler.round(decimals = 3)

        return euler


def myRGB_3(mat):
    r"""
    propose a RGB (red green blue) vector to represent a matrix

    .. note::
        from O Robach

    .. todo:: to be placed in generaltools
    """
    allpermu = DictLT.OpSymArray
    allpermudet1 = np.array([kkk for kkk in allpermu if np.linalg.det(kkk) == 1.0])

    vec = LaueGeom.vec_normalTosurface(mat)
    # allvec = np.dot(allpermudet1,vec)
    # allposvec = allvec[np.all(allvec>=0,axis=1)]
    # # pos of row in allposvec where last element is the higher element of each row
    # hpos = np.where(np.argmax(allposvec, axis=1)==2)[0][0]
    # #print hpos
    # return allposvec[hpos].tolist()

    myframe = np.array([[0, 0, 1], [0, 1, 1], [1, 1, 1]])
    invf = np.linalg.inv(myframe)

    # vec expressed in myframe basis
    vec_in_frame = np.dot(invf, vec)

    # find permutation that lead to 3 positive components
    # TODO: do the permutation before expressing the myframe basis ??
    allvec = np.dot(allpermudet1, vec_in_frame)
    allposvec = allvec[np.all(allvec >= 0, axis=1)]

    if len(np.shape(allposvec)) > 1:
        vecmaxfirstcolumn = np.argmax(allposvec[:, 0])
        allposvec = allposvec[vecmaxfirstcolumn]

    return allposvec


def getMisorientation(
    mat, refAxis=NORMAL_TO_SAMPLE_AXIS, followVector=np.array([0, 0, 1])):
    r"""
    compute an angle of a reflection / an axis

    # default axis normal to surface tilted by  40 degrees from horizontal

    .. todo:: to be placed in generaltools

    """

    # angle between mat.G* and refAxis
    #    norm = np.sqrt(np.dot(followVector, followVector))
    rotGstar = np.dot(mat, followVector)

    norm_rotGstar = np.sqrt(np.dot(rotGstar, rotGstar))

    angle = np.arccos(np.dot(rotGstar, refAxis) / (norm_rotGstar)) / DEG

    return angle


def Matrix_to_RGB_2(mat):
    r"""
    use unique representation of orientation matrix with its euler angles

    .. note::
        from O Robach

    .. todo:: to be placed in generaltools
    """
    normalisation_angles = np.array([180.0, 360, 180.0])
    # normalisation_angles = np.ones(3)

    return calc_Euler_angles(FO.find_lowest_Euler_Angles_matrix(mat)[0]
                                                ) / normalisation_angles + np.array([0.1, 0.1, 0.1])



from __future__ import absolute_import
# -*- coding: utf-8 -*-
"""
#  scripts and example Laue Pattern simulation based on lauecore.py  (new name of laue6.py)
#  J. S. Micha   micha [at] esrf [dot] fr
# version May 2019
#  from LaueTools package
#  http://sourceforge.net/projects/lauetools/

"""
__author__ = "Jean-Sebastien Micha, CRG-IF BM32 @ ESRF"
import sys
import os

# # workaround to launch this script with modules in parent folder
# # whatever the folder it is launched in 
# LTpath = os.path.dirname(os.path.abspath(__file__))
# while not LTpath.endswith('LaueTools'):
#     LTpath = os.path.dirname(LTpath)
# LTpath = os.path.dirname(LTpath)
# if LTpath not in sys.path:
#     sys.path.append(LTpath)

import time
import pickle
import numpy as np

import LaueTools
print("Using LaueToolsFolder: ", os.path.abspath(LaueTools.__file__))

import LaueTools.CrystalParameters as CP
from LaueTools.dict_LaueTools import (dict_Rot,
                                    dict_Materials,
                                    dict_Vect,
                                    dict_Extinc,
                                    CST_ENERGYKEV)

import LaueTools.generaltools as GT
import LaueTools.lauecore as LT
import LaueTools.IOLaueTools as IOLT
import LaueTools.LaueGeometry as LTGeo

DEFAULT_TOP_GEOMETRY = LT.DEFAULT_TOP_GEOMETRY
DEFAULT_DETECTOR_DISTANCE = LT.DEFAULT_DETECTOR_DISTANCE
# --- -------------------  TESTS & EXAMPLES
def test_simulation():
    """
    test scenario
    """
    

    #  SIMULATION Compute list of spots for each grains

    emin = 5
    emax = 25
    # What kind of 2D plot must be display#
    # 'xy' plot (xcam, ycam)  (X horizontal + along the x-ray ; Y horizontal + towards the GMT door; Z vertical + towards the ceilling)
    # 'XYcam' plot (xcam, ycam) Xmas
    # 'xytest' plot pour test orientation cam
    # '2thetachi' plot (2theta, chi)
    # 'directionq' plot (longitude, latitude) de q
    whatplot = "2thetachi"
    # detection geometry:
    # 'Z>0' top camera 'Y>0' side + 'Y<0' side - 'X<0' backreflection  'X>0' transmission
    kf_direction = "Z>0"  # 'Z>0' 'Y>0' 'Y<0' 'X<0'

    # mat_UO2_He_103_1 = GT.OrientMatrix_fromGL(filename = 'UO2_He_103_1L.dat')
    mat_Id = np.eye(3)
    # mymat1 = fromelemangles_toMatrix([  1.73284101e-01 ,  3.89021368e-03 ,  4.53663032e+01])
    # mymat2 = fromelemangles_toMatrix([  0. ,  35.19 ,45.])
    # mattest= [[-0.72419551 , 0.09344237, -0.6832345 ], [-0.59410999 , 0.41294652,  0.69029595], [ 0.3465515 ,  0.90488673, -0.24714785]]
    # #mat_Cu = GT.OrientMatrix_fromGL(filename = 'matrixfromopenGL_311_c2.dat')
    # mat_Cu = np.eye(3)
    # matfromHough = fromelemangles_toMatrix([85., 68., 76.])
    # matfromHough = fromEULERangles_toMatrix([42., 13., 20.])
    grain1 = [mat_Id, "dia", dict_Rot["Si_001"], "Si"]
    # grain2 = [mat_Id, 'dia', mymat2,'Si']
    # #grainUO2_103 = [vecteurref, [1, 1, 1],mat_UO2_He_103_1,'UO2']
    # grainHough = [mat_Id, 'fcc', matfromHough,'UO2']
    # graintest = [mat_Id, 'fcc', mattest,'UO2']

    # GRAIN Must Be Prepared !!! see CP.Prepare_Grain() or fill before dict_Material

    starting_time = time.time()

    # vec and indices of spots Z > 0 (vectors [x, y, z] array, miller indices [H, K, L] array)
    vecind = LT.getLaueSpots(
        CST_ENERGYKEV / emax,
        CST_ENERGYKEV / emin,
        [grain1],
        fastcompute=0,
        kf_direction=kf_direction,
    )
    pre_filename = "totobill"

    linestowrite = [[""]]
    # print "vecind",vecind
    # selecting RR nodes without harmonics (fastcompute = 1 loses the miller indices and RR positions associations for quicker computation)
    oncam_sansh = LT.filterLaueSpots(
        vecind,
        fileOK=1,
        fastcompute=0,
        kf_direction=kf_direction,
        detectordistance=DEFAULT_DETECTOR_DISTANCE,
        HarmonicsRemoval=1,
        linestowrite=linestowrite,
    )
    # print "oncam_sansh",oncam_sansh

    # plot and save and write file of simulated Data
    # third input is a list whose each element corresponds to one grain list of spots object

    # loadind experimental data:
    data_filename = "Ge_test.cor"
    data_2theta, data_chi, data_x, data_y, data_I = IOLT.readfile_cor(data_filename)

    # ----------  Time consumption information
    finishing_time = time.time()
    duration = finishing_time - starting_time
    print("Time duration for computation %.2f sec." % duration)

    Plot_Laue(
        emin,
        emax,
        oncam_sansh,
        data_2theta,
        data_chi,
        data_filename,
        removeharmonics=1,
        kf_direction=kf_direction,
        Plot_Data=1,
        Display_label=0,
        What_to_plot=whatplot,
        saveplotOK=0,
        WriteLogFile=1,
        linestowrite=linestowrite,
    )

    # logbook file edition
    IOLT.writefile_log(
        output_logfile_name=pre_filename + ".log", linestowrite=linestowrite
    )

    return CreateData_(oncam_sansh, outputname="tototable", pickledOK=0), oncam_sansh

def CreateData_(list_spots, outputname="fakedatapickle", pickledOK=0):
    """
    returns list of (2theta, chi)
    
    :param list_spots: list of list of instances of class 'spot'
    
    Data may be pickled if needed

    USED in scripts
    """
    mydata = [[elem.Twicetheta, elem.Chi] for elem in list_spots[0]]
    if outputname != None:
        if pickledOK:
            filepickle = open(outputname, "w")
            pickle.dump(mydata, filepickle)
            filepickle.close()
        else:
            line_to_write = []
            line_to_write.append(["2theta   chi h k l E spotnumber"])
            spotnumber = 0
            for i in list(range(len(list_spots))):

                for elem in list_spots[i]:

                    line_to_write.append(
                        [
                            str(elem.Twicetheta),
                            str(elem.Chi),
                            str(elem.Millers[0]),
                            str(elem.Millers[1]),
                            str(elem.Millers[2]),
                            str(elem.EwaldRadius * CST_ENERGYKEV),
                            str(spotnumber),
                        ]
                    )
                    spotnumber += 1

            filetowrite = open(outputname, "w")
            aecrire = line_to_write
            for line in aecrire:
                lineData = "\t".join(line)
                filetowrite.write(lineData)
                filetowrite.write("\n")
            filetowrite.close()

    return mydata

def test_speed():
    # ----------  Time consumption information
    Starting_time = time.time()

    Array2thetachi, Oncam = test_simulation()

def simulatepattern(
    grain,
    emin,
    emax,
    kf_direction,
    data_filename,
    PlotLaueDiagram=1,
    Plot_Data=0,
    verbose=0,
    detectordistance=DEFAULT_DETECTOR_DISTANCE,
    ResolutionAngstrom=False,
    Display_label=1,
    HarmonicsRemoval=1,
):
    """
    USED only for scripts ...???
    """
    whatplot = "2thetachi"

    print("kf_direction", kf_direction)

    Starting_time = time.time()

    # vec and indices of spots Z > 0 (vectors [x, y, z] array, miller indices [H, K, L] array)

    vecind = LT.getLaueSpots(
        CST_ENERGYKEV / emax,
        CST_ENERGYKEV / emin,
        grain,
        fastcompute=0,
        kf_direction=kf_direction,
        verbose=verbose,
        ResolutionAngstrom=ResolutionAngstrom,
    )
    pre_filename = "laue6simul"

    print("len(vecind[0])", len(vecind[0][0]))
    linestowrite = [[""]]
    # selecting RR nodes without harmonics (fastcompute = 1 loses the miller indices and RR positions associations for quicker computation)
    try:
        oncam_sansh = LT.filterLaueSpots(
            vecind,
            fileOK=1,
            fastcompute=0,
            kf_direction=kf_direction,
            detectordistance=detectordistance,
            HarmonicsRemoval=HarmonicsRemoval,
            linestowrite=linestowrite,
        )
    #        print "oncam_sansh",oncam_sansh
    except UnboundLocalError:
        raise UnboundLocalError("Empty list of spots or vector (variable: vecind)")

    # plot and save and write file of simulated Data
    # third input is a list whose each element corresponds to one grain list of spots object

    data_2theta, data_chi = None, None  # exp. data

    if data_filename != None:
        # loadind experimental data:
        try:
            res = IOLT.readfile_cor(data_filename)

        except IOError:
            print(
                "You must enter the name of experimental datafile (similar to e.g 'Ge_test.cor')"
            )
            print("or set Plot_Data=0")
            return

        else:
            alldata, data_2theta, data_chi, data_x, data_y, data_I, detparam = res

    else:
        print(
            "You must enter the name of experimental datafile (similar to e.g 'Ge_test.cor')"
        )

    # ----------  Time consumption information
    finishing_time = time.time()
    duration = finishing_time - Starting_time
    print("Time duration for computation %.2f sec." % duration)

    if PlotLaueDiagram:

        Plot_Laue(
            emin,
            emax,
            oncam_sansh,
            data_2theta,
            data_chi,
            data_filename,
            removeharmonics=1,
            kf_direction=kf_direction,
            Plot_Data=Plot_Data,
            Display_label=Display_label,
            What_to_plot=whatplot,
            saveplotOK=0,
            WriteLogFile=1,
            linestowrite=linestowrite,
        )
   
    return True

def simulatepurepattern(
    grain,
    emin,
    emax,
    kf_direction,
    data_filename,
    PlotLaueDiagram=1,
    Plot_Data=0,
    verbose=0,
    detectordistance=DEFAULT_DETECTOR_DISTANCE,
    ResolutionAngstrom=False,
    Display_label=1,
    HarmonicsRemoval=1,
):
    """
    NOT USED ??!!
    """
    vecind = LT.getLaueSpots( CST_ENERGYKEV / emax, CST_ENERGYKEV / emin, grain,
                            fastcompute=0, kf_direction=kf_direction,
                            verbose=verbose, ResolutionAngstrom=ResolutionAngstrom, )

    print("len(vecind[0])", len(vecind[0][0]))

    # selecting RR nodes without harmonics (fastcompute = 1 loses the miller indices and RR positions associations for quicker computation)

    oncam_sansh = LT.filterLaueSpots(vecind,
                                    fileOK=0,
                                    fastcompute=0,
                                    kf_direction=kf_direction,
                                    detectordistance=detectordistance,
                                    HarmonicsRemoval=HarmonicsRemoval)

    return True


def Plot_Laue( _emin, _emax, list_of_spots, _data_2theta, _data_chi, _data_filename,
    Plot_Data=1,
    Display_label=1,
    What_to_plot="2thetachi",
    saveplotOK=1,
    WriteLogFile=1,
    removeharmonics=0,
    kf_direction=DEFAULT_TOP_GEOMETRY,
    linestowrite=[[""]]):
    """
    basic function to plot LaueData for showcase
    """
    import annot

    # creation donnee pour etiquette sur figure
    # if plotOK:
    font = {"fontname": "Courier", "color": "k", "fontweight": "normal", "fontsize": 7}
    fig = figure(figsize=(6.0, 6.0), dpi=80)  # ? small but ok for savefig!
    ax = fig.add_subplot(111)

    title(
        "Laue pattern %.1f-%.1f keV \n %s" % (_emin, _emax, _data_filename),
        font,
        fontsize=12,
    )
    # text(0.5, 2.5, 'a line', font, color = 'k')

    if removeharmonics == 0:
        oncam_sansh = Calc_spot_on_cam_sansh(
            list_of_spots, fileOK=WriteLogFile, linestowrite=linestowrite
        )
    elif removeharmonics == 1:
        print("harmonics have been already removed")
        oncam_sansh = list_of_spots

    nb_of_grains = len(list_of_spots)
    print("Number of grains in Plot_Laue", nb_of_grains)

    xxx = emptylists(nb_of_grains)
    yyy = emptylists(nb_of_grains)

    energy = emptylists(nb_of_grains)
    trans = emptylists(nb_of_grains)
    indy = emptylists(nb_of_grains)

    # calculation for a list of spots object of spots belonging to the camera direction

    if WriteLogFile:
        linestowrite.append(["-" * 60])
        linestowrite.append([" Number of spots w / o harmonics: "])

    # building  simul data
    # loop over grains
    for i in list(range(nb_of_grains)):

        print(
            "Number of spots on camera (w / o harmonics): GRAIN number ",
            i,
            " : ",
            len(oncam_sansh[i]),
        )
        if WriteLogFile:
            linestowrite.append(
                [
                    "--- GRAIN number ",
                    str(i),
                    " : ",
                    str(len(oncam_sansh[i])),
                    " ----------",
                ]
            )
        facscale = 1.0
        # loop over spots
        for elem in oncam_sansh[i]:
            if What_to_plot == "2thetachi" and (
                isinstance(kf_direction, list) or kf_direction in ("Z>0", "Y>0", "Y<0")
            ):
                xxx[i].append(elem.Twicetheta)
                yyy[i].append(elem.Chi)
            elif What_to_plot == "XYcam" or kf_direction in ("X>0", "X<0"):
                facscale = 2.0  # for bigger spot on graph
                xxx[i].append(elem.Xcam)
                yyy[i].append(elem.Ycam)
            elif What_to_plot == "projgnom":  # essai pas fait
                xxx[i].append(elem.Xcam)
                yyy[i].append(elem.Ycam)
            #            elif What_to_plot == 'directionq':
            #                xxx[i].append(elem._philongitude)
            #                yyy[i].append(elem._psilatitude)
            else:
                xxx[i].append(elem.Xcam)
                yyy[i].append(elem.Ycam)
            energy[i].append(elem.EwaldRadius * CST_ENERGYKEV)
            # if etiquette: # maintenant avec la souris!!!
            indy[i].append(elem.Millers)

        print(len(xxx[i]))

    # plot of simulation and or not data
    # affichage avec ou sans etiquettes
    dicocolor = {0: "k", 1: "r", 2: "g", 3: "b", 4: "c", 5: "m"}
    #        dicosymbol = {0:'k.',1:'r.',2:'g.',3:'b.',4:'c.',5:'m.'}

    # exp data
    if _data_2theta is not None and _data_chi is not None and Plot_Data:
        xxx_data = tuple(_data_2theta)
        yyy_data = tuple(_data_chi)

        # PLOT EXT DATA
        # print "jkhhkjhkjhjk**********",xxx_data[:20]
        ax.scatter(xxx_data, yyy_data, s=40, c="w", marker="o", faceted=True, alpha=0.5)
        # END PLOT EXP DATA

    # Plot Simulated laue patterns
    for i in list(range(nb_of_grains)):  # loop over grains
        # ax.plot(tuple(xxx[i]),tuple(yyy[i]),dicosymbol[i])

        # display label to the side of the point
        if nb_of_grains >= 3:
            #            trans[i] = offset(ax, fig, 10, -5 * (nb_of_grains - 2) + 5 * i)
            trans[i] = offset(
                ax.transData, fig, 10, -5 * (nb_of_grains - 2) + 5 * i, units="dots"
            )
        else:
            #            trans[i] = offset(ax, fig, 10, 5 * (-1) ** (i % 2))
            trans[i] = offset(ax.transData, fig, 10, 5 * (-1) ** (i % 2), units="dots")

        print("nb of spots for grain # %d : %d" % (i, len(xxx[i])))
        #        print tuple(xxx[i])
        #        print tuple(yyy[i])
        ax.scatter(
            tuple(xxx[i]),
            tuple(yyy[i]),
            s=[facscale * int(200.0 / en) for en in energy[i]],
            c=dicocolor[(i + 1) % (len(dicocolor))],
            marker="o",
            faceted=False,
            alpha=0.5,
        )

        if Display_label:  # displaying labels c and d at position a, b
            for a, b, c, d in zip(
                tuple(xxx[i]), tuple(yyy[i]), tuple(indy[i]), tuple(energy[i])
            ):
                ax.text(
                    a,
                    b,
                    "%s,%.1f" % (str(c), d),
                    font,
                    fontsize=7,
                    color=dicocolor[i % (len(dicocolor))],
                    transform=trans[i],
                )

    ax.set_aspect(aspect="equal")
    if What_to_plot == "XYcam":
        # ax.set_axis_off()
        ax.set_xlim((0.0, 2030.0))
        ax.set_ylim((0.0, 2030.0))
    #    elif What_to_plot == 'directionq':
    #        #ax.set_axis_off()
    #        ax.set_xlim((120.,240.))
    #        ax.set_ylim((20.,70.))

    # CCD circle contour border drawing

    t = np.arange(0.0, 6.38, 0.1)

    if What_to_plot == "2thetachi" and kf_direction in ("Y>0", "Y<0"):
        if kf_direction == "Y>0":
            si = 1
        else:
            si = -1

        circY = si * (-90 + 45 * np.sin(t))
        circX = 90 + 45 * np.cos(t)
        Xtol = 5.0
        Ytol = 5.0

    elif What_to_plot == "2thetachi" and (
        isinstance(kf_direction, list) or kf_direction in ("Z>0")
    ):

        circX = 90 + 45 * np.cos(t)
        circY = 45 * np.sin(t)
        Xtol = 5.0
        Ytol = 5.0
    #    elif What_to_plot == 'directionq': #essai coordonnee du cercle camera???
    #        circX = map(lambda tutu: 180 + 60 * math.cos(tutu), t)
    #        circY = map(lambda tutu: 45 + 25 * math.sin(tutu), t)
    #        Xtol = 5.
    #        Ytol = 5.
    # TODO embellished with numpy like above
    elif kf_direction in ("X>0", "X<0"):
        circX = [1024 + 1024 * math.cos(tutu) for tutu in t]
        circY = [1024 + 1024 * math.sin(tutu) for tutu in t]
        Xtol = 20
        Ytol = 20
    else:
        circX = [1024 + 1024 * math.cos(tutu) for tutu in t]
        circY = [1024 + 1024 * math.sin(tutu) for tutu in t]
        Xtol = 20
        Ytol = 20
    ax.plot(circX, circY, "")

    if saveplotOK:
        fig.savefig(
            "figlaue.png", dpi=300, facecolor="w", edgecolor="w", orientation="portrait"
        )

    if not Display_label:
        whole_xxx = xxx[0]
        whole_yyy = yyy[0]
        whole_indy = indy[0]
        whole_energy = energy[0]
        for i in list(range(nb_of_grains - 1)):
            whole_xxx += xxx[i + 1]
            whole_yyy += yyy[i + 1]
            whole_indy += indy[i + 1]
            whole_energy += energy[i + 1]

        whole_energy_r = np.around(np.array(whole_energy), decimals=2)

        todisplay = np.vstack((np.array(whole_indy, dtype=np.int8).T, whole_energy_r)).T

        # af =  annot.AnnoteFinder(xxx[0] + xxx[1],yyy[0] + yyy[1],indy[0] + indy[1])
        # af =  annot.AnnoteFinder(whole_xxx, whole_yyy, whole_indy, xtol = 5.,ytol = 5.)
        # af =  annot.AnnoteFinder(whole_xxx, whole_yyy, whole_energy_r, xtol = 10.,ytol = 10.)
        af = annot.AnnoteFinder(whole_xxx, whole_yyy, todisplay, xtol=Xtol, ytol=Ytol)
        # af =  AnnoteFinder(xxx[0],yyy[0],energy[0],indy[0])
        connect("button_press_event", af)
    show()


def Calc_spot_on_cam_sansh(listoflistofspots, fileOK=0, linestowrite=[[""]]):

    """ Calculates list of grains spots on camera w / o harmonics,
    from liligrains (list of grains spots instances scattering in 2pi steradians)
    [[spots grain 0],[spots grain 0],etc] = >
    [[spots grain 0],[spots grain 0],etc] w / o harmonics and on camera  CCD

    TODO: rather useless  only used in Plot_Laue(), need to be replace by filterLaueSpots
    """

    nbofgrains = len(listoflistofspots)

    _oncam = emptylists(nbofgrains)  # list of spot on the camera (harmonics included)
    _oncamsansh = emptylists(
        nbofgrains
    )  # list of spot on the camera (harmonics EXCLUDED) only fondamental (but still with E > Emin)

    if fileOK:
        linestowrite.append(["\n"])
        linestowrite.append(
            ["--------------------------------------------------------"]
        )
        linestowrite.append(
            ["------------- Simulation Data --------------------------"]
        )
        linestowrite.append(
            ["--------------------------------------------------------"]
        )
        linestowrite.append(["\n"])
        linestowrite.append(
            [
                "#grain, h, k, l, energy(keV), 2theta (deg), chi (deg), X_Xmas, Y_Xmas, X_JSM, Y_JSM, Xtest, Ytest"
            ]
        )

    for i in list(range(nbofgrains)):
        condCam = (elem.Xcam ** 2 + elem.Ycam ** 2 <= (DEFAULT_DETECTOR_DIAMETER / 2) ** 2)
        _oncam[i] = [elem for elem in listoflistofspots[i] if (elem.Xcam is not None and condCam)]

        # Creating list of spot without harmonics
        _invdict = {}  # for each grain, use of _invdict to remove harmonics present in _oncam

        for elem in sorted(_oncam[i], reverse=True):
            _invdict[elem.__hash__()] = elem
            _oncamsansh[i] = [_invdict[cle] for cle in list(_invdict.keys())]

        if fileOK:
            IOLT.Writefile_data_log(_oncamsansh[i], i, linestowrite=linestowrite)

    return _oncamsansh


emin = 5
emax = 15
kf_direction = "Z>0"
data_filename = "Ge_test.cor"  # experimental data

Id = np.eye(3)

if 0:
    grainSi = CP.Prepare_Grain("Si", dict_Rot["Si_001"])

    grainSi[2] = GT.fromEULERangles_toMatrix([20.0, 10.0, 50.0])

    print("\n*******************\nSimulation with fastcompute = 0")
    vecind = LT.getLaueSpots(
        CST_ENERGYKEV / emax,
        CST_ENERGYKEV / emin,
        [grainSi],
        fastcompute=0,
        kf_direction=kf_direction,
        verbose=1,
    )

    print("nb of spots with harmonics %d\n\n" % len(vecind[0][0]))

    print("\n*********\nSimulation with fastcompute = 1")

    vecindfast = LT.getLaueSpots(CST_ENERGYKEV / emax,
                                CST_ENERGYKEV / emin,
                                [grainSi],
                                fastcompute=1,
                                kf_direction=kf_direction,
                                verbose=1)

    print("nb of spots with harmonics fast method", len(vecindfast[0][0]))

    print("\n*******************\n --------  Harmonics removal -----------\n")
    linestowrite = [[""]]
    oncam_sansh00 = LT.filterLaueSpots(
        vecind,
        fileOK=1,
        fastcompute=0,
        kf_direction=kf_direction,
        HarmonicsRemoval=1,
        linestowrite=linestowrite,
    )

    print("after harm removal 00", len(oncam_sansh00[0]))

    oncam_sansh01 = LT.filterLaueSpots(
        vecind,
        fileOK=1,
        fastcompute=1,
        kf_direction=kf_direction,
        HarmonicsRemoval=1,
        linestowrite=linestowrite,
    )

    print("after harm removal 01", len(oncam_sansh01[0]))

    oncam_sansh11 = LT.filterLaueSpots(
        vecindfast,
        fileOK=1,
        fastcompute=1,
        kf_direction=kf_direction,
        HarmonicsRemoval=1,
        linestowrite=linestowrite,
    )

    print("after harm removal 11", len(oncam_sansh11[0]))

    # ------------------------
    print("\n*****************************\n rax compute\n")
    grainSi[2] = GT.fromEULERangles_toMatrix([20.0, 10.0, 50.0])

    # res = get_2thetaChi_withoutHarmonics(grainSi, 5, 15)

if 1:  # some grains example
    inittime = time.time()

    emax = 50
    emin = 5

    elem1 = "Ti"
    elem2 = "Cu"

    kf_direction = "Z>0"

    BmatTi = CP.calc_B_RR(dict_Materials["Ti"][1])

    BmatCu = CP.calc_B_RR(dict_Materials["Cu"][1])

    Umat = dict_Rot["OrientSurf111"]

    grainTi = [BmatTi, "no", Umat, "Ti"]

    grainCu = [BmatCu, "fcc", Id, "Cu"]
    grainCu__ = [BmatCu, "no", Id, "Cu"]

    dict_Materials["Cu2"] = ["Cu2", [5, 3.6, 3.6, 90, 90, 90], "fcc"]
    BmatCu2 = CP.calc_B_RR(dict_Materials["Cu2"][1])

    dict_Materials["Cu3"] = ["Cu3", [Id, Umat, Id, BmatCu2], "fcc"]  # Da, U, Dc, B

    grainCu2 = CP.Prepare_Grain("Cu2", Id)
    grainCu3 = CP.Prepare_Grain("Cu3", "ihjiĥiohio")

    grainSi1 = CP.Prepare_Grain("Cu", Id)
    grainSi2 = CP.Prepare_Grain("Si", dict_Rot["OrientSurf111"])

    grainAl2o3 = CP.Prepare_Grain("Al2O3", Id)

    # grains= [grainSi,grainCu3]
    grains = [grainSi1, grainCu3, grainAl2o3]

    print(grains)

    simulatepattern(
        grains,
        emin,
        emax,
        kf_direction,
        data_filename,
        Plot_Data=0,
        verbose=1,
        HarmonicsRemoval=0,
    )

    finaltime = time.time()

    print("computation time is ", finaltime - inittime)

if 0:
    # strain study: ---------------------------------------
    dict_Materials["Cu2"] = ["Cu2", [5, 3.6, 3.6, 90, 90, 90], "fcc"]
    BmatCu2 = CP.calc_B_RR(dict_Materials["Cu2"][1])

    Umat = dict_Rot["OrientSurf111"]
    # Da, U, Dc, B
    dict_Materials["Cu3"] = ["Cu3", [Id, Umat, Id, BmatCu2], "fcc"]

    grains = []
    for k in list(range(-5, 6)):
        dict_Materials["Cu_%d" % k] = dict_Materials["Cu3"]
        # set material label
        dict_Materials["Cu_%d" % k][0] = "Cu_%d" % k
        # set deformation in absolute space
        dict_Materials["Cu_%d" % k][1][0] = [
            [1 + 0.01 * k, 0, 0],
            [0, 1, 0],
            [0, 0, 1.0],
        ]
        grains.append(CP.Prepare_Grain("Cu_%d" % k))

    simulatepattern(grains, emin, emax, kf_direction, data_filename, Plot_Data=0)
    # -----------------------------------------------

if 0:

    Bmat = dict_Vect["543_909075"]
    Umat = dict_Rot["OrientSurf001"]
    Dc = [[1.02, 0.01, 0], [-0.01, 0.98, 0.005], [0.001, -0.02, 1.01]]
    Dc = [[1.00, 0.00, 0], [-0.00, 1.00, 0.000], [0.000, -0.00, 1.03]]

    dict_Materials["mycell"] = ["mycell", [Id, Umat, Id, Bmat], "fcc"]
    dict_Materials["mycell_strained"] = ["mycell_strained", [Id, Umat, Dc, Bmat], "fcc"]

    mygrain = CP.Prepare_Grain("mycell")
    mygrain_s = CP.Prepare_Grain("mycell_strained")

    grains = [mygrain, mygrain_s]

    simulatepattern(grains, emin, emax, kf_direction, data_filename, Plot_Data=0)

if 0:

    Bmat = dict_Vect["543_909075"]
    Umat = dict_Rot["mat311c1"]
    Dc = dict_Vect["shear4"]

    dict_Materials["mycell_s"] = ["mycell_s", [Id, Umat, Dc, Bmat], "fcc"]
    dict_Materials["mycell"] = ["mycell", [Id, Umat, Id, Bmat], "fcc"]

    mygrain_s = CP.Prepare_Grain("mycell_s")
    mygrain = CP.Prepare_Grain("mycell")

    grains = [mygrain_s]

    simulatepattern(
        grains, emin, emax, kf_direction, data_filename, Plot_Data=0, verbose=1
    )

if 0:
    emin = 5
    emax = 22
    kf_direction = "X>0"  # transmission
    kf_direction = "Z>0"  # reflection

    ResolutionAngstrom = 2.0

    # overwriting dict_Materials['smallpro']
    dict_Materials["smallpro"] = ["smallpro", [20, 4.8, 49, 90, 90, 90], "no"]
    mygrain = CP.Prepare_Grain("smallpro", dict_Rot["mat311c1"])
    mygrain = CP.Prepare_Grain("smallpro", Id)

    grains = [mygrain]

    simulatepattern(
        grains,
        emin,
        emax,
        kf_direction,
        data_filename,
        Plot_Data=0,
        verbose=1,
        detectordistance=69,
        ResolutionAngstrom=ResolutionAngstrom,
    )

if 0:
    emin = 5
    emax = 30
    kf_direction = "X>0"  # transmission
    #        kf_direction = 'Z>0' # reflection

    ResolutionAngstrom = False

    # overwriting dict_Materials['smallpro']

    mat111alongx = np.dot(GT.matRot([1, 0, 0], -45.0), GT.matRot([0, 0, 1], 45.0))

    print("mat111alongx", mat111alongx)

    matmono = np.dot(GT.matRot([1, 0, 0], -1), mat111alongx)

    mygrain = CP.Prepare_Grain("Cu", matmono)

    grains = [mygrain]

    simulatepattern(
        grains,
        emin,
        emax,
        kf_direction,
        data_filename,
        Plot_Data=0,
        verbose=1,
        detectordistance=10,
        ResolutionAngstrom=ResolutionAngstrom,
    )

if 0:
    emin = 8
    emax = 25
    kf_direction = "X<0"  # back reflection
    #        kf_direction = 'Z>0' # reflection

    ResolutionAngstrom = False

    # overwriting dict_Materials['smallpro']

    matmono = np.dot(GT.matRot([0, 0, 1], 20), np.eye(3))

    mygrain = CP.Prepare_Grain("Si", matmono)

    grains = [mygrain]

    simulatepattern(
        grains,
        emin,
        emax,
        kf_direction,
        data_filename,
        Plot_Data=0,
        verbose=1,
        detectordistance=55,
        ResolutionAngstrom=ResolutionAngstrom,
        Display_label=0,
    )

if 0:
    emin = 8
    emax = 25
    kf_direction = "X<0"  # back reflection
    #        kf_direction = 'Z>0' # reflection

    ResolutionAngstrom = False

    # overwriting dict_Materials['smallpro']

    matmother = np.dot(GT.matRot([0, 0, 1], -5), np.eye(3))
    matmisorient = np.dot(GT.matRot([-1, 1, 1], 0.1), matmother)

    maingrain = CP.Prepare_Grain("Si", matmother)

    mygrain = CP.Prepare_Grain("Si", matmisorient)

    grains = [maingrain, mygrain]

    simulatepattern(
        grains,
        emin,
        emax,
        kf_direction,
        data_filename,
        Plot_Data=0,
        verbose=1,
        detectordistance=55,
        ResolutionAngstrom=ResolutionAngstrom,
        Display_label=0,
    )

if 0:
    emin = 5
    emax = 20
    kf_direction = [90, 45]
    #        kf_direction = 'Z>0' # reflection

    ResolutionAngstrom = False

    # overwriting dict_Materials['smallpro']

    matmother = np.dot(GT.matRot([0, 0, 1], 0), np.eye(3))
    #        matmisorient = np.dot(GT.matRot([-1, 1, 1], .1), matmother)

    maingrain = CP.Prepare_Grain("Si", matmother)

    grains = [maingrain, maingrain]

    simulatepattern(
        grains,
        emin,
        emax,
        kf_direction,
        data_filename,
        Plot_Data=0,
        verbose=1,
        detectordistance=70,
        ResolutionAngstrom=ResolutionAngstrom,
        Display_label=0,
    )

if 0:
    emin = 5
    emax = 20
    #        kf_direction = [0, 0]
    kf_direction = "X>0"  #

    ResolutionAngstrom = False

    # overwriting dict_Materials['smallpro']

    matmother = np.dot(GT.matRot([0, 0, 1], 0), np.eye(3))
    #        matmisorient = np.dot(GT.matRot([-1, 1, 1], .1), matmother)

    maingrain = CP.Prepare_Grain("Si", matmother)

    grains = [maingrain, maingrain]

    simulatepattern(
        grains,
        emin,
        emax,
        kf_direction,
        data_filename,
        Plot_Data=0,
        verbose=1,
        detectordistance=70,
        ResolutionAngstrom=ResolutionAngstrom,
        Display_label=0,
    )


if 0:
    emin = 5
    emax = 22

    Detpos = 0  # 1 = 'top' 0 = 'trans'

    if Detpos == 1:
        # on top
        kf_direction = "Z>0"  # reflection
        Detdist = 70  # mm
    elif Detpos == 0:
        # transmission
        kf_direction = "X>0"  # transmission
        Detdist = 100  # mm

    ResolutionAngstrom = None

    # overwriting dict_Materials['smallpro']
    dict_Materials["smallpro"] = ["smallpro", [20, 4.8, 49, 90, 90, 90], "no"]
    mygrain = CP.Prepare_Grain("smallpro", dict_Rot["mat311c1"])
    mygrain = CP.Prepare_Grain("smallpro", Id)
    ResolutionAngstrom = 2.0

    grains = [mygrain]

    simulatepattern(
        grains,
        emin,
        emax,
        kf_direction,
        data_filename,
        Plot_Data=0,
        verbose=1,
        detectordistance=Detdist,
        ResolutionAngstrom=ResolutionAngstrom,
    )

if 0:
    emin = 5
    emax = 22

    Detpos = 0  # 1 = 'top' 0 = 'trans'

    if Detpos == 1:
        # on top
        kf_direction = "Z>0"  # reflection
        Detdist = 70  # mm
    elif Detpos == 0:
        # transmission
        kf_direction = "X>0"  # transmission
        Detdist = 100.0  # mm

    ResolutionAngstrom = None

    # overwriting dict_Materials['smallpro']
    dict_Materials["smallpro"] = ["smallpro", [20, 4.8, 49, 90, 90, 90], "no"]
    ResolutionAngstrom = 2.0

    for ori in list(dict_Rot.keys())[2:3]:
        mygrain = CP.Prepare_Grain("smallpro", dict_Rot[ori])

        grains = [mygrain]

        mydata = simulatepattern(
            grains,
            emin,
            emax,
            kf_direction,
            data_filename,
            Plot_Data=0,
            verbose=1,
            detectordistance=Detdist,
            ResolutionAngstrom=ResolutionAngstrom,
        )

    ard = np.array(mydata[0])

    import pylab as pp

    pp.figure(1)
    pp.scatter(ard[:, 0], ard[:, 1])

    xyd = np.array([[elem.Xcam, elem.Ycam] for elem in mydata[1][0]])

    pp.figure(2)
    pp.scatter(xyd[:, 0], xyd[:, 1])

    calib = [100.0, 1024.0, 1024.0, 90, 0.0]

    xyd_fromfind2 = LTGeo.calc_xycam_from2thetachi(
        ard[:, 0],
        ard[:, 1],
        calib,
        verbose=0,
        pixelsize=165.0 / 2048,
        kf_direction=kf_direction)

    X, Y, theta = xyd_fromfind2

    pp.scatter(X, Y, c="r")

    pp.show()


if 0:
    emin = 50
    emax = 120

    Detpos = 0  # 1 = 'top' 0 = 'trans'

    if Detpos == 1:
        # on top
        kf_direction = "Z>0"  # reflection
        Detdist = 70  # mm
    elif Detpos == 0:
        # transmission
        kf_direction = "X>0"  # transmission
        Detdist = 100.0  # mm

    ResolutionAngstrom = 0.5

    for ori in list(dict_Rot.keys())[2:3]:
        mygrain = CP.Prepare_Grain("Ni", dict_Rot[ori])

        grains = [mygrain]

        mydata = simulatepattern(
            grains,
            emin,
            emax,
            kf_direction,
            data_filename,
            Plot_Data=0,
            verbose=1,
            detectordistance=Detdist,
            ResolutionAngstrom=ResolutionAngstrom,
        )

    ard = np.array(mydata[0])

    print("mydata", mydata)

    import pylab as pp

    pp.figure(1)
    pp.scatter(ard[:, 0], ard[:, 1])

    xyd = np.array([[elem.Xcam, elem.Ycam] for elem in mydata[1][0]])

    miller = [elem.Millers for elem in mydata[1][0]]

    print(miller)

    pp.figure(2)
    pp.scatter(xyd[:, 0], xyd[:, 1])

    calib = [105.624, 1017.50, 996.62, -0.027, -116.282]
    pixelsize = 0.048

    xyd_fromfind2 = LTGeo.calc_xycam_from2thetachi(
        ard[:, 0],
        ard[:, 1],
        calib,
        verbose=0,
        pixelsize=pixelsize,
        kf_direction=kf_direction)

    X, Y, theta = xyd_fromfind2

    pp.scatter(X, Y, c="r")

    pp.show()

    f = open("Ni_fake_transmission.dat", "w")
    f.write("X Y I from simulation\n")
    for k in list(range(len(X))):
        f.write(
            "%s %s %s %s %s %s %s %s %s %s %s\n"
            % (
                X[k],
                Y[k],
                65000 * (1 - 0.8 * k / len(X)),
                65000 * (1 - 0.8 * k / len(X)),
                1.47,
                1.81,
                82.460,
                -1.07,
                1.78,
                624.74,
                64740,
            )
        )
    f.write("# %s pixelsize %s" % (str(calib), pixelsize))
    f.close()

if 0:  # Si 111 in transmission on ID15
    emin = 80
    emax = 100

    Detpos = 0  # 1 = 'top' 0 = 'trans'

    if Detpos == 1:
        # on top
        kf_direction = "Z>0"  # reflection
        Detdist = 100  # mm
    elif Detpos == 0:
        # transmission
        kf_direction = "X>0"  # transmission
        Detdist = 100.0  # mm

    mattrans3 = [
        [0.998222982873332, 0.04592237603705288, -0.037973831023250665],
        [0.0036229001244100726, 0.5893105150537007, 0.8078985031808321],
        [0.05947899678171664, -0.8066004291012069, 0.5880969279936651],
    ]
    mygrain = CP.Prepare_Grain("Si", dict_Rot["mat111alongx"])
    mygrain = CP.Prepare_Grain("Si", mattrans3)

    grains = [mygrain]

    mydata = simulatepattern(
        grains,
        emin,
        emax,
        kf_direction,
        data_filename,
        PlotLaueDiagram=0,
        Plot_Data=0,
        verbose=1,
        detectordistance=Detdist,
        ResolutionAngstrom=None,
    )

    # two theta chi
    ard = np.array(mydata[0])

    print("ard", ard)

    import pylab as pp

    pp.figure(1)
    pp.scatter(ard[:, 0], ard[:, 1])

    # pixel X Y position with default camera settings ...
    #         xyd = np.array([[elem.Xcam, elem.Ycam] for elem in mydata[1][0]])
    #
    #         pp.figure(2)
    #         pp.scatter(xyd[:, 0], xyd[:, 1])

    calib = [105.0, 1024.0, 1024.0, 0.0, 0.0]

    xyd_fromfind2 = LTGeo.calc_xycam_from2thetachi(
        ard[:, 0],
        ard[:, 1],
        calib,
        verbose=0,
        pixelsize=0.048,
        kf_direction=kf_direction)

    X, Y, theta = xyd_fromfind2

    intensity = np.ones_like(X)

    IOLT.writefile_Peaklist(
        "ID15transSi111",
        np.array(
            [
                X,
                Y,
                intensity,
                intensity,
                intensity,
                intensity,
                intensity,
                intensity,
                intensity,
                intensity,
                intensity,
            ]
        ).T,
    )

    pp.figure(3)
    pp.scatter(X, Y, c="r")

    pp.show()



"""
module containing a GUI class to compute lattice parameters in both spaces (real and reciprocal)
"""
import os
import sys
import re

import numpy as np

import wx
if sys.version_info.major == 3:
    from .. import CrystalParameters as CP
else:
    import CrystalParameters as CP

# --- ---------------------   UB Matrix Editor
class B0MatrixEditor(wx.Frame):
    """
    GUI class to compute lattice parameters in both spaces (real and reciprocal)
    """
    def __init__(self, parent, _id, title):

        wx.Frame.__init__(self, parent, _id, title, size=(720, 1000))

        self.dirname = os.getcwd()
        self.parent = parent
        # variables
        self.modify = False
        self.last_name_saved = ""
        self.last_name_stored = ""
        self.replace = False

        self.CurrentMat = "Default"
        font3 = wx.Font(10, wx.MODERN, wx.NORMAL, wx.BOLD)

        self.panel = wx.Panel(self, -1, style=wx.SIMPLE_BORDER, size=(715, 995), pos=(5, 5))

        dr = wx.StaticText(self.panel, -1, "Direct Space", (60, 10))
        dr.SetFont(font3)

        self.rbeditor = wx.RadioButton(self.panel, -1, "Text Editor Input", (25, 40), style=wx.RB_GROUP)
        wx.StaticText(self.panel, -1, "[[#,#,#],[#,#,#],[#,#,#]]", (25, 60))
        self.text = wx.TextCtrl(self.panel, 1000, "", pos=(25, 85), size=(250, 90),
                                        style=wx.TE_MULTILINE | wx.TE_PROCESS_ENTER)
        self.text.SetFocus()

        self.rbelem = wx.RadioButton(self.panel, -1, "Matrix Elements Input", (25, 190))

        self.mat_a11 = wx.TextCtrl(self.panel, -1, "1", (20, 220), size=(80, -1))
        self.mat_a12 = wx.TextCtrl(self.panel, -1, "0", (120, 220), size=(80, -1))
        self.mat_a13 = wx.TextCtrl(self.panel, -1, "0", (220, 220), size=(80, -1))
        self.mat_a21 = wx.TextCtrl(self.panel, -1, "0", (20, 250), size=(80, -1))
        self.mat_a22 = wx.TextCtrl(self.panel, -1, "1", (120, 250), size=(80, -1))
        self.mat_a23 = wx.TextCtrl(self.panel, -1, "0", (220, 250), size=(80, -1))
        self.mat_a31 = wx.TextCtrl(self.panel, -1, "0", (20, 280), size=(80, -1))
        self.mat_a32 = wx.TextCtrl(self.panel, -1, "0", (120, 280), size=(80, -1))
        self.mat_a33 = wx.TextCtrl(self.panel, -1, "1", (220, 280), size=(80, -1))

        self.rblatticeparam_direct = wx.RadioButton(self.panel, -1, "Real Lattice parameters Input", (25, 320))
        wx.StaticText(self.panel, -1, "a", (40, 350))
        self.a = wx.TextCtrl(self.panel, -1, "1", (20, 380), size=(60, -1))
        wx.StaticText(self.panel, -1, "b", (120, 350))
        self.b = wx.TextCtrl(self.panel, -1, "1", (100, 380), size=(60, -1))
        wx.StaticText(self.panel, -1, "c", (200, 350))
        self.c = wx.TextCtrl(self.panel, -1, "1", (180, 380), size=(60, -1))
        wx.StaticText(self.panel, -1, "Angst.", (260, 385))
        wx.StaticText(self.panel, -1, "alpha", (20, 410))
        self.alpha = wx.TextCtrl(self.panel, -1, "90", (20, 440), size=(60, -1))
        wx.StaticText(self.panel, -1, "beta", (100, 410))
        self.beta = wx.TextCtrl(self.panel, -1, "90", (100, 440), size=(60, -1))
        wx.StaticText(self.panel, -1, "gamma", (180, 410))
        self.gamma = wx.TextCtrl(self.panel, -1, "90", (180, 440), size=(60, -1))
        wx.StaticText(self.panel, -1, "deg.", (260, 445))

        # reciprocal part
        # letter 's' at the end of widgets name stands for 'star' corresponding to reciprocal space variable

        hshift = 350
        drs = wx.StaticText(self.panel, -1, "Reciprocal Space", (hshift + 60, 10))
        drs.SetFont(font3)

        self.rbeditors = wx.RadioButton(self.panel, -1, "Text Editor Input Bmatrix", (hshift + 25, 40))
        wx.StaticText(self.panel, -1, "[[#,#,#],[#,#,#],[#,#,#]]", (hshift + 25, 60))
        self.texts = wx.TextCtrl(self.panel, 1003, "", pos=(hshift + 25, 85),
                            size=(250, 90), style=wx.TE_MULTILINE | wx.TE_PROCESS_ENTER)
        self.texts.SetFocus()

        self.rbelems = wx.RadioButton(self.panel, -1, "B0 Matrix Elements Input", (hshift + 25, 190))

        self.mat_a11s = wx.TextCtrl(self.panel, -1, "1", (hshift + 20, 220), size=(80, -1))
        self.mat_a12s = wx.TextCtrl(self.panel, -1, "0", (hshift + 120, 220), size=(80, -1))
        self.mat_a13s = wx.TextCtrl(self.panel, -1, "0", (hshift + 220, 220), size=(80, -1))
        self.mat_a21s = wx.TextCtrl(self.panel, -1, "0", (hshift + 20, 250), size=(80, -1))
        self.mat_a22s = wx.TextCtrl(self.panel, -1, "1", (hshift + 120, 250), size=(80, -1))
        self.mat_a23s = wx.TextCtrl(self.panel, -1, "0", (hshift + 220, 250), size=(80, -1))
        self.mat_a31s = wx.TextCtrl(self.panel, -1, "0", (hshift + 20, 280), size=(80, -1))
        self.mat_a32s = wx.TextCtrl(self.panel, -1, "0", (hshift + 120, 280), size=(80, -1))
        self.mat_a33s = wx.TextCtrl(self.panel, -1, "1", (hshift + 220, 280), size=(80, -1))

        self.rblatticeparam_reciprocal = wx.RadioButton(self.panel, -1, "Reciprocal Lattice parameters Input",
                                                                            (hshift + 25, 320))
        wx.StaticText(self.panel, -1, "a*", (hshift + 40, 350))
        self.astar = wx.TextCtrl(self.panel, -1, "1", (hshift + 20, 380), size=(60, -1))
        wx.StaticText(self.panel, -1, "b*", (hshift + 120, 350))
        self.bstar = wx.TextCtrl(self.panel, -1, "1", (hshift + 100, 380), size=(60, -1))
        wx.StaticText(self.panel, -1, "c*", (hshift + 200, 350))
        self.cstar = wx.TextCtrl(self.panel, -1, "1", (hshift + 180, 380), size=(60, -1))
        wx.StaticText(self.panel, -1, "1/Angst.", (hshift + 260, 385))
        wx.StaticText(self.panel, -1, "alpha*", (hshift + 20, 410))
        self.alphastar = wx.TextCtrl(self.panel, -1, "90", (hshift + 20, 440), size=(60, -1))
        wx.StaticText(self.panel, -1, "beta*", (hshift + 100, 410))
        self.betastar = wx.TextCtrl(self.panel, -1, "90", (hshift + 100, 440), size=(60, -1))
        wx.StaticText(self.panel, -1, "gamma*", (hshift + 180, 410))
        self.gammastar = wx.TextCtrl(self.panel, -1, "90", (hshift + 180, 440), size=(60, -1))
        wx.StaticText(self.panel, -1, "deg.", (hshift + 260, 445))

        self.rbeditors.SetValue(True)

        buttonConvert = wx.Button(self.panel, 105, "Compute && Convert", pos=(150, 480), size=(400, 50))
        # buttonload.SetFont(font3)

        self.text.Bind(wx.EVT_TEXT, self.OnTextChanged, id=1000)
        self.text.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        self.texts.Bind(wx.EVT_TEXT, self.OnTextChanged, id=1003)
        self.texts.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        buttonConvert.Bind(wx.EVT_BUTTON, self.OnConvert, id=105)

        #tips
        tiprealB = 'Columns are real unit cell vector basis, a,b,c expressed in LaueTools frame\n'
        tiprealB += '[ ax   bx  cx]\n'
        tiprealB += '[ ay   by  cy]\n'
        tiprealB += '[ az   bz  cz]\n'
        dr.SetToolTipString(tiprealB)
        self.rbeditor.SetToolTipString(tiprealB)
        self.rbelem.SetToolTipString(tiprealB)

        # listtctrl = [self.mat_a11,self.mat_a12,self.mat_a13,
        #             self.mat_a11,self.mat_a11,self.mat_a11,
        #             self.mat_a11,self.mat_a11,self.mat_a11]

        tiprecipB = 'B0 matrix: Columns are reciprocal unit cell vector basis, a*,b*,c* expressed in LaueTools frame\n'
        tiprecipB += '[ a*x   b*x  c*x]\n'
        tiprecipB += '[ a*y   b*y  c*y]\n'
        tiprecipB += '[ a*z   b*z  c*z]\n'
        drs.SetToolTipString(tiprecipB)
        self.rbeditors.SetToolTipString(tiprecipB)
        self.rbelems.SetToolTipString(tiprecipB)

        buttonConvert.SetToolTipString('Compute lattice parameters and matrices in both spaces')

        #-------------- GUI middle and second part ----------------------------------
        wx.StaticText(self.panel, -1, "_" * 90, (25, 530))

        self.Build_list_of_B()
        vshift = 80
        buttonread = wx.Button(self.panel, 101, "Look", pos=(20, vshift + 500), size=(60, 25))
        # buttonread.SetFont(font3)

        self.comboRot = wx.ComboBox(self.panel, 6, "Identity", (100, vshift + 500), choices=self.list_of_Vect)

        wx.StaticText(self.panel, -1, "in B0 matrix", (300, vshift + 500 + 5))

        buttonsave = wx.Button(self.panel, 102, "Save", pos=(20, vshift + 540), size=(60, 25))
        # buttonsave.SetFont(font3)

        wx.StaticText(self.panel, -1, "B0 matrix in", (110, vshift + 545))
        self.filenamesave = wx.TextCtrl(self.panel, -1, "*.b0mat", (200, vshift + 540), size=(100, 25))
        wx.StaticText(self.panel, -1, "(on hard disk)", (300, vshift + 545))

        buttonstore = wx.Button(self.panel, 103, "Store", pos=(20, vshift + 580), size=(60, 25))
        # buttonstore.SetFont(font3)
        wx.StaticText(self.panel, -1, "B0 matrix in", (110, vshift + 585))
        self.filenamestore = wx.TextCtrl(self.panel, -1, "", (200, vshift + 580), size=(100, 25))
        wx.StaticText(self.panel, -1, "(will appear in a*,b*,c* simulation menu)", (300, vshift + 585))

        buttonload = wx.Button(self.panel, 104, "Load", pos=(20, vshift + 620), size=(60, 25))
        # buttonload.SetFont(font3)
        wx.StaticText(self.panel, -1, "B0 Matrix from simple ASCII file",
                                                                    (100, vshift + 625))

        buttonread.Bind(wx.EVT_BUTTON, self.OnLookMatrix, id=101)
        self.comboRot.Bind(wx.EVT_COMBOBOX, self.EnterComboRot, id=6)
        buttonsave.Bind(wx.EVT_BUTTON, self.OnSaveFile, id=102)
        buttonstore.Bind(wx.EVT_BUTTON, self.OnStoreFile, id=103)
        buttonload.Bind(wx.EVT_BUTTON, self.DoOpenFile, id=104)

        # -------------    Lower and third GUI part   ----------------
        self.vshift2 = vshift + 535 + 60

        wx.StaticText(self.panel, -1, "_" * 90, (25, self.vshift2 + 50))

        st000 = wx.StaticText(self.panel, -1, "Crystal Unit Cell", (250, self.vshift2 + 70))
        st000.SetFont(font3)

        """
        q =   Da U Dc B  G*
        Da deformation in lab frame(or compute from D expressed in sample frame):  Da = I + strain_a
        strain_a is general strain neither symetric nor antisymetric, contain pure strain + pure rigid body rotation

        Dc deformation in crystal frame  a*,b*,c* Dc = I + strain_c
        strain_c is general strain neither symetric nor antisymetric, contain pure strain + pure rigid body rotation

        U is orient matrix in lab frame
        B is Bmatrix whose each colum is a*,b*,c* expressed in lab frame. Bmatrix can contain rotation part
        ( can different from triangular up matrix)
        """

        self.stepx = 120
        postext = 90
        pos0x = 70

        wx.StaticText(self.panel, -1, "q   = ", (10, self.vshift2 + postext))
        wx.StaticText(self.panel, -1, "Da       .", (pos0x, self.vshift2 + postext))
        wx.StaticText(self.panel, -1, "U        .", (pos0x + 1 * self.stepx, self.vshift2 + postext))
        wx.StaticText(self.panel, -1, "B        .", (pos0x + 2 * self.stepx, self.vshift2 + postext))
        wx.StaticText(self.panel, -1, "Dc       .", (pos0x + 3 * self.stepx, self.vshift2 + postext))
        wx.StaticText(self.panel, -1, "G*", (pos0x + 3 * self.stepx + 100, self.vshift2 + postext))
        wx.StaticText(self.panel, -1, "Extinc", (pos0x + 3 * self.stepx + 150, self.vshift2 + postext))

        self.poscombos = 120

        # combos of unit cell reference parameters
        self.DisplayCombosUnitCell()

        posv = (self.poscombos + self.vshift2) + 40

        storeCellbtn = wx.Button(self.panel, -1, "Store Unit as new material", pos=(20, posv), size=(80, 25))
        wx.StaticText(self.panel, -1, "in", (110, posv + 5))
        self.namestore = wx.TextCtrl(self.panel, -1, "", (150, posv), size=(100, 25))
        wx.StaticText(self.panel, -1, "(Will appear in Elem list for classical indexation)", (280, posv + 5))

        quitbtn = wx.Button(self.panel, -1, "Quit", (575, self.vshift2 + 180), size=(120, 100))

        quitbtn.Bind(wx.EVT_BUTTON, self.OnQuit)
        storeCellbtn.Bind(wx.EVT_BUTTON, self.OnStoreRefCell)

        #---tooltips
        tipstore = 'Store in LaueToolsGUI a new element from Real lattice parameter above'
        storeCellbtn.SetToolTipString(tipstore)
        self.namestore.SetToolTipString(tipstore)

        self.StatusBar()

    def Build_list_of_B(self):
        # building list of choices of B matrix

        List_Vect_name = list(self.parent.dict_Vect.keys())
        List_Vect_name.remove("Default")
        List_Vect_name.sort()

        self.list_of_Vect = ["Default"] + List_Vect_name

    def DisplayCombosUnitCell(self):

        self.Build_list_of_B()

        List_U_name = list(self.parent.dict_Rot.keys())
        List_U_name.remove("Identity")
        List_U_name.sort()

        self.list_of_Rot = ["Identity"] + List_U_name

        List_Transform_name = list(self.parent.dict_Transforms.keys())
        List_Transform_name.remove("Identity")
        List_Transform_name.sort()

        self.list_of_Strain = ["Identity"] + List_Transform_name

        self.comboDa = wx.ComboBox(self.panel, -1, "Identity", (50, self.vshift2 + self.poscombos),
                                                choices=self.list_of_Strain, size=(100, -1))
        self.comboU = wx.ComboBox(self.panel, -1, "Identity", (50 + self.stepx, self.vshift2 + self.poscombos),
                                                choices=self.list_of_Rot, size=(100, -1))
        self.comboB = wx.ComboBox(self.panel, -1, "Identity", (50 + 2 * self.stepx, self.vshift2 + self.poscombos),
                                                choices=self.list_of_Vect, size=(100, -1))
        self.comboDc = wx.ComboBox(self.panel, -1, "Identity", (50 + 3 * self.stepx, self.vshift2 + self.poscombos),
                                                choices=self.list_of_Strain, size=(100, -1))
        self.comboExtinc = wx.ComboBox(self.panel, -1, "NoExtinction", (50 + 4 * self.stepx + 40, self.vshift2 + self.poscombos),
                                    choices=list(self.parent.dict_Extinc.keys()), size=(100, -1))

    def OnStoreRefCell(self, _):
        key_material = str(self.namestore.GetValue())
        # factor structure peak extinction
        struct_extinc = self.parent.dict_Extinc[self.comboExtinc.GetValue()]

        # UnitCellParameters is either [a,b,c,alpha,beta,gamma] or list of 4 matrices [Da,U,Dc,B]
        Da = self.parent.dict_Transforms[self.comboDa.GetValue()]
        U = self.parent.dict_Rot[self.comboU.GetValue()]
        B = self.parent.dict_Vect[self.comboB.GetValue()]
        Dc = self.parent.dict_Transforms[self.comboDc.GetValue()]

        UnitCellParameters = [Da, U, B, Dc]

        UnitCellParameters = []
        for par_txtctrl in [self.a, self.b, self.c, self.alpha, self.beta, self.gamma]:
            UnitCellParameters.append(float(par_txtctrl.GetValue()))

        print("UnitCellParameters", UnitCellParameters)
        print("Stored key_material", [key_material, UnitCellParameters, struct_extinc])
        self.parent.dict_Materials[key_material] = [key_material, UnitCellParameters, struct_extinc]

    def StatusBar(self):
        self.statusbar = self.CreateStatusBar()
        self.statusbar.SetFieldsCount(3)
        self.statusbar.SetStatusWidths([-5, -2, -1])

    def ToggleStatusBar(self, _):
        if self.statusbar.IsShown():
            self.statusbar.Hide()
        else:
            self.statusbar.Show()

    def OnTextChanged(self, evt):
        self.modify = True
        evt.Skip()

    def OnKeyDown(self, evt):
        keycode = evt.GetKeyCode()
        if keycode == wx.WXK_INSERT:
            if not self.replace:
                self.statusbar.SetStatusText("INS", 2)
                self.replace = True
            else:
                self.statusbar.SetStatusText("", 2)
                self.replace = False
        evt.Skip()

    def OnSaveFile(self, _):
        """
        Saves the matrix in ASCII file on Hard Disk from the ASCII editor or the 9 input elements
        """
        self.last_name_saved = self.filenamesave.GetValue()

        if self.last_name_saved:

            try:
                if self.rbeditor.GetValue():
                    paramraw = self.texts.GetValue()

                    listval = re.split("[ ()\[\)\;\,\]\n\t\a\b\f\r\v]", paramraw)
                    #             print "listval", listval
                    listelem = []
                    for elem in listval:
                        try:
                            val = float(elem)
                            listelem.append(val)
                        except ValueError:
                            continue

                    nbval = len(listelem)
                    if nbval != 9:
                        txt = "Something wrong, I can't read this matrix %s \n."
                        txt += "It doesn't contain 9 elements with float type ..."%paramraw
                        print(txt)

                        wx.MessageBox(txt, "VALUE ERROR")
                        return

                    allm = []
                    ind_elem = 0
                    for _ in range(3):
                        for _ in range(3):
                            floatval = listelem[ind_elem]
                            allm.append(floatval)
                            ind_elem += 1

                else:
                    m11 = float(self.mat_a11s.GetValue())
                    m12 = float(self.mat_a12s.GetValue())
                    m13 = float(self.mat_a13s.GetValue())

                    m21 = float(self.mat_a21s.GetValue())
                    m22 = float(self.mat_a22s.GetValue())
                    m23 = float(self.mat_a23s.GetValue())

                    m31 = float(self.mat_a31s.GetValue())
                    m32 = float(self.mat_a32s.GetValue())
                    m33 = float(self.mat_a33s.GetValue())

                    allm = (m11, m12, m13, m21, m22, m23, m31, m32, m33)

                f = open(self.last_name_saved, "w")
                text = ("[[%.17f,%.17f,%.17f],\n[%.17f,%.17f,%.17f],\n[%.17f,%.17f,%.17f]]" % allm)
                f.write(text)
                f.close()

                self.statusbar.SetStatusText(os.path.basename(self.last_name_saved) + " saved", 0)
                self.modify = False
                self.statusbar.SetStatusText("", 1)

                fullname = os.path.join(os.getcwd(), self.last_name_saved)
                wx.MessageBox("Matrix saved in %s" % fullname, "INFO")

            except IOError as error:

                dlg = wx.MessageDialog(self, "Error saving file\n" + str(error))
                dlg.ShowModal()
        else:
            wx.MessageBox("Please provide a name for the matrix file to be saved...!", "VALUE ERROR")
            return

    def OnStoreFile(self, _):
        """
        Stores the matrix from the ASCII editor or the 9 entried elements in main list of orientation matrix for further simulation
        """
        self.last_name_stored = self.filenamestore.GetValue()

        if self.last_name_stored:

            if not self.rbeditor.GetValue():  # read 9 elements
                m11 = float(self.mat_a11s.GetValue())
                m12 = float(self.mat_a12s.GetValue())
                m13 = float(self.mat_a13s.GetValue())

                m21 = float(self.mat_a21s.GetValue())
                m22 = float(self.mat_a22s.GetValue())
                m23 = float(self.mat_a23s.GetValue())

                m31 = float(self.mat_a31s.GetValue())
                m32 = float(self.mat_a32s.GetValue())
                m33 = float(self.mat_a33s.GetValue())

                self.parent.dict_Vect[self.last_name_stored] = [[m11, m12, m13],
                                                                [m21, m22, m23],
                                                                [m31, m32, m33]]

            else:  # read ASCII editor
                paramraw = str(self.texts.GetValue())

                listval = re.split("[ ()\[\)\;\,\]\n\t\a\b\f\r\v]", paramraw)
                #             print "listval", listval
                listelem = []
                for elem in listval:
                    try:
                        val = float(elem)
                        listelem.append(val)
                    except ValueError:
                        continue

                nbval = len(listelem)
                if nbval != 9:
                    txt = "Something wrong, I can't read this matrix %s \n."
                    txt += "It doesn't contain 9 elements with float type ..."%paramraw
                    print(txt)

                    wx.MessageBox(txt, "VALUE ERROR")
                    return

                mat = np.zeros((3, 3))
                ind_elem = 0
                for i in range(3):
                    for j in range(3):
                        floatval = listelem[ind_elem]
                        mat[i][j] = floatval
                        ind_elem += 1

                self.parent.dict_Rot[self.last_name_stored] = mat

            self.statusbar.SetStatusText(os.path.basename(self.last_name_stored) + " stored", 0)
            self.DisplayCombosUnitCell()

        else:
            print("No name input !!!")

    def DoOpenFile(self, _):
        wcd = "All files(*)|*|Matrix files(*.b0mat)|*.b0mat"
        _dir = os.getcwd()
        open_dlg = wx.FileDialog(self,
                                    message="Choose a file",
                                    defaultDir=_dir,
                                    defaultFile="",
                                    wildcard=wcd,
                                    style=wx.OPEN | wx.CHANGE_DIR)
        if open_dlg.ShowModal() == wx.ID_OK:
            path = open_dlg.GetPath()

            try:
                _file = open(path, "r")
                text = _file.readlines()
                _file.close()

                if self.texts.GetLastPosition():
                    self.texts.Clear()
                strmat = ""
                for line in text:
                    strmat += line[:-1]
                self.text.WriteText(strmat + "]")
                self.last_name_saved = path
                self.statusbar.SetStatusText("", 1)
                self.modify = False

            except IOError as error:
                dlg = wx.MessageDialog(self, "Error opening file\n" + str(error))
                dlg.ShowModal()

            except UnicodeDecodeError as error:
                dlg = wx.MessageDialog(self, "Error opening file\n" + str(error))
                dlg.ShowModal()

        open_dlg.Destroy()

    def EnterComboRot(self, evt):
        """
        in UB matrix editor
        """
        item = evt.GetSelection()
        self.CurrentMat = self.list_of_Rot[item]
        evt.Skip()

    # def EnterComboDa(self, evt):
    # """
    # in UB matrix editor

    # """
    # item = evt.GetSelection()
    # self.CurrentMat = self.list_of_Rot[item]
    # evt.Skip()

    # def EnterComboU(self, evt):
    # """
    # in UB matrix editor

    # """
    # item = evt.GetSelection()
    # self.CurrentMat = self.list_of_Rot[item]
    # evt.Skip()

    # def EnterComboDc(self, evt):
    # """
    # in UB matrix editor

    # """
    # item = evt.GetSelection()
    # self.CurrentMat = self.list_of_Rot[item]
    # evt.Skip()

    # def EnterComboB(self, evt):
    # """
    # in UB matrix editor

    # """
    # item = evt.GetSelection()
    # self.CurrentMat = self.list_of_Rot[item]
    # evt.Skip()

    # def EntercomboExtinc(self, evt):
    # """
    # in UB matrix editor

    # """
    # item = evt.GetSelection()
    # self.CurrentMat = self.list_of_Rot[item]
    # evt.Skip()

    def OnLookMatrix(self, _):
        matrix = self.parent.dict_Vect[self.CurrentMat]
        print("%s is :" % self.CurrentMat)
        print(matrix)

        self.mat_a11s.SetValue(str(matrix[0][0]))
        self.mat_a12s.SetValue(str(matrix[0][1]))
        self.mat_a13s.SetValue(str(matrix[0][2]))
        self.mat_a21s.SetValue(str(matrix[1][0]))
        self.mat_a22s.SetValue(str(matrix[1][1]))
        self.mat_a23s.SetValue(str(matrix[1][2]))
        self.mat_a31s.SetValue(str(matrix[2][0]))
        self.mat_a32s.SetValue(str(matrix[2][1]))
        self.mat_a33s.SetValue(str(matrix[2][2]))

        self.texts.SetValue(str(matrix))

    def Matrix_from_texteditor(self, texteditor):
        """
        read matrix element from text editor in [[#,#,#],[#,#,#],[#,#,#]] format
        return matrix(array type)
        """
        text = str(texteditor.GetValue())
        tu = text.replace("[", "").replace("]", "")
        ta = tu.split(",")
        try:
            to = [float(elem) for elem in ta]
        except ValueError:
            wx.MessageBox(
                "Text Editor input Bmatrix seems empty. Fill it or Fill others fields and select the associated button, and then click on Convert", "INFO")
            return None
        matrix = np.array([to[:3], to[3:6], to[6:]])
        return matrix

    def Set_lattice_parameter(self, sixvalues, sixtextctrl):
        """
        from six lattice parameters fill the six txtctrls of sixtextctrl
        """
        for k, val in enumerate(sixvalues):
            sixtextctrl[k].SetValue(str(val))

    def Set_matrix_parameter(self, ninevalues, ninetextctrl):
        """
        from nine matrix elements fill the nine txtctrls of ninetextctrl
        """
        for k, val in enumerate(ninevalues):
            ninetextctrl[k].SetValue(str(val))

    def Read_matrix_parameter(self, ninetextctrl):
        """
        read nine matrix elements from ninetextctrl
        """
        mat = []
        for txtcontrol in ninetextctrl:
            mat.append(float(txtcontrol.GetValue()))
        return np.reshape(np.array(mat), (3, 3))

    def Read_lattice_parameter(self, sixtextctrl):
        """
        read six lattice parameters from sixtextctrl
        """
        mat = []
        for txtcontrol in sixtextctrl:
            mat.append(float(txtcontrol.GetValue()))
        return np.array(mat, dtype=float)

    def OnConvert(self, _):
        """
        compute matrices and lattice parameters both in direct and reciprocical
        spaces from data input
        """
        rbuttons = [self.rbeditor,
                    self.rbelem,
                    self.rblatticeparam_direct,
                    self.rbeditors,
                    self.rbelems,
                    self.rblatticeparam_reciprocal]

        txtctrl_lattice_reciprocal = [self.astar,
                                    self.bstar,
                                    self.cstar,
                                    self.alphastar,
                                    self.betastar,
                                    self.gammastar]
        txtctrl_lattice_direct = [self.a,
                                self.b,
                                self.c,
                                self.alpha,
                                self.beta,
                                self.gamma]

        txtctrl_matdirect = [self.mat_a11,
                            self.mat_a12,
                            self.mat_a13,
                            self.mat_a21,
                            self.mat_a22,
                            self.mat_a23,
                            self.mat_a31,
                            self.mat_a32,
                            self.mat_a33]
        txtctrl_matrecip = [self.mat_a11s,
                            self.mat_a12s,
                            self.mat_a13s,
                            self.mat_a21s,
                            self.mat_a22s,
                            self.mat_a23s,
                            self.mat_a31s,
                            self.mat_a32s,
                            self.mat_a33s]

        # rbuttons_state = [elem.GetValue() for elem in rbuttons]

        if rbuttons[0].GetValue():  # self.rbeditor

            BMatrix_direct = self.Matrix_from_texteditor(self.text)
            # strange need of this line otherwise :
            # UnboundLocalError: local variable 'Bmatrix_direct' referenced before assignment
            machin = BMatrix_direct
            truc = np.ravel(machin)
            self.Set_matrix_parameter(truc, txtctrl_matdirect)
            lattice_parameter_direct = CP.matrix_to_rlat(BMatrix_direct)
            self.Set_lattice_parameter(lattice_parameter_direct, txtctrl_lattice_direct)
            # reciprocal space
            Bmatrix = CP.calc_B_RR(lattice_parameter_direct, directspace=1)
            self.texts.SetValue(str(Bmatrix.tolist()))
            self.Set_matrix_parameter(np.ravel(Bmatrix), txtctrl_matrecip)
            lattice_parameter_reciprocal = CP.matrix_to_rlat(Bmatrix)
            self.Set_lattice_parameter(
                lattice_parameter_reciprocal, txtctrl_lattice_reciprocal)

        elif rbuttons[1].GetValue():  # self.rbelem

            BMatrix_direct = self.Read_matrix_parameter(txtctrl_matdirect)
            # strange need of this line otherwise :
            # UnboundLocalError: local variable 'Bmatrix_direct' referenced before assignment
            truc = str(BMatrix_direct.tolist())
            self.text.SetValue(truc)
            lattice_parameter_direct = CP.matrix_to_rlat(BMatrix_direct)
            self.Set_lattice_parameter(lattice_parameter_direct, txtctrl_lattice_direct)
            # reciprocal space
            Bmatrix = CP.calc_B_RR(lattice_parameter_direct, directspace=1)
            self.texts.SetValue(str(Bmatrix.tolist()))
            self.Set_matrix_parameter(np.ravel(Bmatrix), txtctrl_matrecip)
            lattice_parameter_reciprocal = CP.matrix_to_rlat(Bmatrix)
            self.Set_lattice_parameter(
                lattice_parameter_reciprocal, txtctrl_lattice_reciprocal
            )

        elif rbuttons[2].GetValue():  # self.rblatticeparam_direct
            # TODO limitations in angle to have a direct triedral ? 15 90 160 deg => Nan!!
            lattice_parameter_direct = self.Read_lattice_parameter(
                txtctrl_lattice_direct
            )
            Bmatrix_direct = CP.calc_B_RR(lattice_parameter_direct, directspace=0)
            self.text.SetValue(str(Bmatrix_direct.tolist()))
            self.Set_matrix_parameter(np.ravel(Bmatrix_direct), txtctrl_matdirect)
            # reciprocal space
            Bmatrix = CP.calc_B_RR(lattice_parameter_direct, directspace=1)
            self.texts.SetValue(str(Bmatrix.tolist()))
            self.Set_matrix_parameter(np.ravel(Bmatrix), txtctrl_matrecip)
            lattice_parameter_reciprocal = CP.matrix_to_rlat(Bmatrix)
            self.Set_lattice_parameter(
                lattice_parameter_reciprocal, txtctrl_lattice_reciprocal
            )

        elif rbuttons[3].GetValue():  # self.rbeditors
            # start from list(3*3 elements) of B matrix in RS
            Bmatrix = self.Matrix_from_texteditor(self.texts)
            self.Set_matrix_parameter(np.ravel(Bmatrix), txtctrl_matrecip)
            lattice_parameter_reciprocal = CP.matrix_to_rlat(Bmatrix)
            self.Set_lattice_parameter(
                lattice_parameter_reciprocal, txtctrl_lattice_reciprocal
            )
            # direct space
            lattice_parameter_direct = CP.dlat_to_rlat(lattice_parameter_reciprocal)
            self.Set_lattice_parameter(lattice_parameter_direct, txtctrl_lattice_direct)
            Bmatrix_direct = CP.calc_B_RR(lattice_parameter_direct, directspace=0)
            self.text.SetValue(str(Bmatrix_direct.tolist()))
            self.Set_matrix_parameter(np.ravel(Bmatrix_direct), txtctrl_matdirect)

        elif rbuttons[4].GetValue():  # self.rbelems
            # start from 9 entered elements of B matrix in RS
            BMatrix = self.Read_matrix_parameter(txtctrl_matrecip)
            self.texts.SetValue(str(BMatrix.tolist()))
            lattice_parameter_reciprocal = CP.matrix_to_rlat(BMatrix)
            self.Set_lattice_parameter(
                lattice_parameter_reciprocal, txtctrl_lattice_reciprocal
            )
            # direct space
            lattice_parameter_direct = CP.dlat_to_rlat(lattice_parameter_reciprocal)
            self.Set_lattice_parameter(lattice_parameter_direct, txtctrl_lattice_direct)
            Bmatrix_direct = CP.calc_B_RR(lattice_parameter_direct, directspace=0)
            self.text.SetValue(str(Bmatrix_direct.tolist()))
            self.Set_matrix_parameter(np.ravel(Bmatrix_direct), txtctrl_matdirect)

        elif rbuttons[5].GetValue():  # self.rblatticeparam_reciprocal
            lattice_parameter_reciprocal = self.Read_lattice_parameter(
                txtctrl_lattice_reciprocal
            )
            Bmatrix = CP.calc_B_RR(lattice_parameter_reciprocal, directspace=0)
            self.texts.SetValue(str(Bmatrix.tolist()))
            self.Set_matrix_parameter(np.ravel(Bmatrix), txtctrl_matrecip)
            # direct space
            lattice_parameter_direct = CP.dlat_to_rlat(lattice_parameter_reciprocal)
            self.Set_lattice_parameter(lattice_parameter_direct, txtctrl_lattice_direct)
            Bmatrix_direct = CP.calc_B_RR(lattice_parameter_direct, directspace=0)
            self.text.SetValue(str(Bmatrix_direct.tolist()))
            self.Set_matrix_parameter(np.ravel(Bmatrix_direct), txtctrl_matdirect)

    def OnQuit(self, evt):
        dlg = wx.MessageDialog(
            self, 'To use stored UB Matrices, click on "refresh choices" button before.'
        )
        if dlg.ShowModal() == wx.ID_OK:
            self.Close()
            evt.Skip()
        else:
            evt.Skip()

if __name__ == "__main__":
    import dict_LaueTools as DictLT

    class parentB0Editor(wx.Frame):
        """gui class to test
        """
        def __init__(self, parent, _id, title):

            wx.Frame.__init__(self, parent, _id, title)
            self.dict_Vect = DictLT.dict_Vect
            self.dict_Transforms = DictLT.dict_Transforms
            self.dict_Extinc = DictLT.dict_Extinc
            self.dict_Rot = DictLT.dict_Rot
            self.dict_Materials = DictLT.dict_Materials

            Editorframe = B0MatrixEditor(
                    self, -1, "editor"
                )
            Editorframe.Show(True)

    EditorApp = wx.App()
    GUIFrame = parentB0Editor(
        None,
        -1,
        "BO matrix and lattice parameters computer")

    GUIFrame.Show()

    EditorApp.MainLoop()
